@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2007, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -37,8 +37,10 @@
 /////////////////////////////////////////////////////////////////////////////////////////
 // Output macros and app exit codes
 /////////////////////////////////////////////////////////////////////////////////////////
-#define ERROR_OUT(system,msg)  ( fflush(stdout), fprintf(stderr, "%s ERROR: %s\n", system, msg), fflush(stderr) )
-#define WARNING_OUT(msg)       ( fflush(stdout), fprintf(stderr, "Warning: %s\n", msg), fflush(stderr) ) 
+#define WRITE_STDERR(type,sys,msg) ( fflush(stdout), fprintf(stderr, "*** %-5s: (%s): %s\n", (type),(sys),(msg)), fflush(stderr) )
+#define ERROR_OUT(sys,msg)   WRITE_STDERR("ERROR", (sys), (msg))
+#define INFO_OUT(sys,msg)    WRITE_STDERR("Info", (sys), (msg))
+#define WARNING_OUT(sys,msg) WRITE_STDERR("Warn", (sys), (msg))
 
 // exit codes
 #define S_SATISFIABLE   10  // problem is satisfiable
@@ -50,7 +52,44 @@
 // Clasp::Application
 /////////////////////////////////////////////////////////////////////////////////////////
 namespace Clasp {
-	
+class WriteLemmas {
+public:
+	WriteLemmas(std::ostream& os);
+	~WriteLemmas();
+	void attach(SharedContext& ctx);
+	void detach();
+	void flush(Constraint_t::Set types, uint32 maxLbd);
+	bool unary(Literal, Literal) const;
+	bool binary(Literal, Literal, Literal) const;
+private:
+	SharedContext* ctx_;
+	std::ostream&  os_;
+	mutable uint32 outShort_;
+};
+/////////////////////////////////////////////////////////////////////////////////////////
+// Clasp specific app options
+/////////////////////////////////////////////////////////////////////////////////////////
+struct ClaspAppOptions {
+	ClaspAppOptions();
+	typedef std::pair<uint32, uint32> QPair;
+	typedef ProgramOptions::StringSeq StringSeq;
+	static bool mappedOpts(ClaspAppOptions*, const std::string&, const std::string&);
+	void initOptions(ProgramOptions::OptionContext& root);
+	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	StringSeq   input;     // list of input files - only first used!
+	std::string lemmaOut;  // optional file name for writing learnt lemmas
+	std::string lemmaIn;   // optional file name for reading learnt lemmas
+	QPair       quiet;     // configure printing of models and optimization values
+	uint32      verbose;   // verbosity level
+	uint32      timeout;   // timeout in seconds (default: none=-1)
+	uint32      stats;     // print statistics
+	uint32      outf;      // output format
+	char        ifs;       // output field separator
+	uint8       outLbd;    // optional lbd limit for lemma out
+	uint8       inLbd;     // optional lbd for lemma in
+	bool        fastExit;  // force fast exit (no dtors)
+	enum OutputFormat { out_def = 0, out_comp = 1, out_json = 2 };
+};
 class Application : public ProgramOptions::AppOptions, public ClaspFacade::Callback {
 public:
 	static Application& instance();    // returns the singleton instance
@@ -65,20 +104,16 @@ private:
 	const Application& operator=(const Application&);
 	// -------------------------------------------------------------------------------------------
 	// AppOptions interface
-	void printHelp(const ProgramOptions::OptionContext& root)    ;
-	void printVersion(const ProgramOptions::OptionContext& root) ;
-	void initOptions(ProgramOptions::OptionContext& root) {
+	void    printHelp(const ProgramOptions::OptionContext& root)    ;
+	void    printVersion(const ProgramOptions::OptionContext& root) ;
+	HelpOpt initHelpOption() const;
+	void    initOptions(ProgramOptions::OptionContext& root) {
 		claspre_.initOptions(root);
-		cmdOpts_.initOptions(root, config_);
-		ProgramOptions::OptionGroup basic("Basic Options");
-		basic.addOptions()
-			("lemma-out" , ProgramOptions::storeTo(lemmaOut_)->arg("<file>"), "Write learnt lemmas to %A on exit")
-			("lemma-in"  , ProgramOptions::storeTo(lemmaIn_)->arg("<file>"), "Read additional clauses from %A")
-		;
-		root.add(basic);
+		clasp_.initOptions(root, config_);
+		app_.initOptions(root);
 	}
-	bool validateOptions(const ProgramOptions::ParsedOptions& vm, ProgramOptions::Messages& m) {
-		return claspre_.validateOptions(vm) && cmdOpts_.validateOptions(vm, m);
+	bool    validateOptions(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions& vm, ProgramOptions::Messages& m) {
+		return app_.validateOptions(vm, m) && claspre_.validateOptions(vm) && clasp_.validateOptions(root, vm, m);
 	}
 	// -------------------------------------------------------------------------------------------
 	// ClaspFacade::Callback interface
@@ -90,29 +125,36 @@ private:
 	std::istream& getStream();
 	void killAlarm();
 	void kill(int sig);
+	int  blockSignals();
+	void unblockSignals(bool deliverPending);
 	void readLemmas();
-	void writeLemmas();
 	int  exception(int status, const char* what);
-	void prepareExit() { fflush(stdout); fflush(stderr);  }
+	void appTerminate(int exitCode) const;
+	void printDefaultConfigs() const;
 	// -------------------------------------------------------------------------------------------  
 	// Status information & output
 	void configureOutput(Input::Format f);
 	void model(const Solver& s, const Enumerator& e, bool cons);
 	void printResult(int sig);
 	// -------------------------------------------------------------------------------------------  
+	typedef PodVector<const SolveStats*>::type StatsVec;
 	ClaspConfig                   config_;
-	ClaspOptions                  cmdOpts_;
+	ClaspAppOptions               app_;
+	ClaspOptions                  clasp_;
 	Claspre::Options              claspre_;
 	Timer<ProcessTime>            cpuTotalTime_;
 	Timer<RealTime>               timer_[ClaspFacade::num_states]; // one for each state
 	double                        timeToFirst_;                    // time to first model
 	double                        timeToLast_;                     // time to last model
-	std::string                   lemmaOut_;
-	std::string                   lemmaIn_;
+	StatsVec                      stats_;
 	ClaspFacade*                  facade_;
+	std::auto_ptr<WriteLemmas>    writeLemmas_;
 	std::auto_ptr<OutputFormat>   out_;
-	volatile sig_atomic_t         stopped_;
+	volatile sig_atomic_t         blocked_;
+	volatile sig_atomic_t         pending_;
 };
 
+const char* const clasp_app_banner = "clasp version " CLASP_VERSION;
+
 }
 #endif
