@@ -246,7 +246,7 @@ bool GeneralOptions::mapSatPre(GeneralOptions* this_, const std::string&, const
 	GlobalOptions* global = this_->config;
 	std::auto_ptr<SatElite::SatElite> pre(new SatElite::SatElite());
 	uint32* pos = &pre->options.maxIters, *end = pos + (sizeof(pre->options)/sizeof(uint32)); 
-	if (!parseSequence<uint32>(StringSlice(value), pos, end - pos, 0)) return false;
+	if (!parseSequence<uint32>(StringSlice(value), pos, uint32(end - pos), 0)) return false;
 	if (pre->options.maxIters) { global->ctx.satPrepro.reset(pre.release()); }
 	return true;
 }
@@ -648,14 +648,14 @@ bool ClaspOptions::applyDefaultConfig(const ProgramOptions::OptionContext& root,
 	// check if command-line is valid
 	if (!mode->validateOptions(root, vm, m) || !search->validateOptions(root, vm, m)) { return false; }
 	OptionConfig def(mode->defConfig);
+	if (search->solverOpts()->search == Clasp::SolverStrategies::no_learning) {
+		if      (vm.count("configuration") == 0){ def = OptionConfig("[NoLearn]: --heuristic=unit"); }
+		else if (vm.count("heuristic") == 0)    { removeConfig(def, "heuristic"); def.cmdLine.append(" --heuristic=unit"); }
+	}
 	if (vm.count("deletion") && search->enabledDel != 3) {
 		if ((search->enabledDel & 1) == 0) { removeConfig(def, "del-grow"); }
 		if ((search->enabledDel & 2) == 0) { removeConfig(def, "del-cfl");  }
 	}
-	if (search->solverOpts()->search == Clasp::SolverStrategies::no_learning && vm.count("heuristic") == 0) {
-		removeConfig(def, "heuristic");
-		def.cmdLine.append(" --heuristic=unit");
-	}
 	// all from command line
 	ProgramOptions::ParsedOptions parsed(vm);
 	// add options from selected default config
