@@ -320,7 +320,7 @@ void DefaultOutput::reportState(int state, bool enter, double time) {
 }	
 
 void DefaultOutput::printModel(const Model& m, const SymbolTable& index, const Enumerator& en) {
-	comment(1, "Answer: %"PRIu64"\n", en.enumerated);
+	comment(1, "Answer: %" PRIu64"\n", en.enumerated);
 	printf("%s", format_[cat_value]);
 	const char* sep = "";
 	if (index.type() == SymbolTable::map_indirect) {
@@ -372,10 +372,10 @@ void DefaultOutput::printOptimize(const SharedMinimizeData& m) {
 void DefaultOutput::printOptimizeValues(const SharedMinimizeData& m) const {
 	const SharedMinimizeData::SumVec& opt = m.optimum()->opt;
 	const char* sep = "";
-	printf("%s%"PRId64, sep, opt[0]);
+	printf("%s%" PRId64, sep, opt[0]);
 	sep = IFS_ != '\n' ? as_ : " ";
 	for (uint32 i = 1, end = m.numRules(); i != end; ++i) {
-		printf("%s%"PRId64, sep, opt[i]);
+		printf("%s%" PRId64, sep, opt[i]);
 	}
 }
 
@@ -389,7 +389,7 @@ void DefaultOutput::printResult(const RunSummary& sol, const SolveStats** stats,
 		printKey("Models");
 		if (!sol.complete()) {
 			char buf[64];
-			int wr    = sprintf(buf, "%"PRIu64, sol.models());
+			int wr    = sprintf(buf, "%" PRIu64, sol.models());
 			buf[wr]   = '+';
 			buf[wr+1] = 0;
 			printf("%-6s\n", buf);
@@ -399,7 +399,7 @@ void DefaultOutput::printResult(const RunSummary& sol, const SolveStats** stats,
 		}
 		if (en.enumerated) {
 			if (en.enumerated != sol.models()) { 
-				printKeyValue("  Enumerated", "%"PRIu64"\n", en.enumerated);
+				printKeyValue("  Enumerated", "%" PRIu64"\n", en.enumerated);
 			}
 			if (sol.consequences) {
 				printf("%s  %-*s: %s\n", format_[cat_comment], w_-2, sol.consequences, sol.complete()?"yes":"unknown");
@@ -443,11 +443,11 @@ void DefaultOutput::printStats(const RunSummary& sol, const SolveStats** stats,
 }
 
 void DefaultOutput::printSolveStats(const SolveStats& st) const {
-	printKeyValue("Choices", "%"PRIu64"\n", st.choices);
-	printKeyValue("Conflicts", "%"PRIu64"\n", st.conflicts);
+	printKeyValue("Choices", "%" PRIu64"\n", st.choices);
+	printKeyValue("Conflicts", "%" PRIu64"\n", st.conflicts);
 	printKeyValue("Restarts", "%-6"PRIu64"", st.restarts);
 	if (st.restarts) {
-		printf(" (Average: %.2f Last: %"PRIu64")", average(st.analyzed, st.restarts), st.cflLast);
+		printf(" (Average: %.2f Last: %" PRIu64")", average(st.analyzed, st.restarts), st.cflLast);
 	}
 	printf("\n");
 }
@@ -517,9 +517,9 @@ void DefaultOutput::printJumpStats(const SolveStats& st) const {
 		printf("\n");
 		printKeyValue("Backtracks", "%-6"PRIu64"\n", st.conflicts-st.analyzed);
 		printKeyValue("Backjumps", "%-6"PRIu64, st.analyzed);
-		printf(" (Bounded: %"PRIu64")\n", js.bJumps);
+		printf(" (Bounded: %" PRIu64")\n", js.bJumps);
 		printKeyValue("Skippable Levels", "%-6"PRIu64, js.jumpSum);
-		printf(" (Skipped: %"PRIu64" Rate: %5.1f%%)\n", js.jumpSum - js.boundSum, percent(js.jumpSum - js.boundSum, js.jumpSum));
+		printf(" (Skipped: %" PRIu64" Rate: %5.1f%%)\n", js.jumpSum - js.boundSum, percent(js.jumpSum - js.boundSum, js.jumpSum));
 		printKeyValue("Max Jump Length", "%-6u", js.maxJump);
 		printf(" (Executed: %u)\n", js.maxJumpEx);
 		printKeyValue("Max Bound Length", "%-6u\n", js.maxBound);
@@ -555,15 +555,15 @@ void DefaultOutput::printParallelStats(const SolveStats& stats, bool accu) const
 		uint64 sumLem = stats.learnts[0] + stats.learnts[1];
 		if (!accu) {
 			printKeyValue("CPU Time",  "%.3fs\n", ps.cpuTime);
-			printKeyValue("Models", "%"PRIu64"\n", stats.models);
+			printKeyValue("Models", "%" PRIu64"\n", stats.models);
 		}
 		printKeyValue("Distributed", "%-6"PRIu64, ps.distributed);
 		printf(" (Ratio: %.2f Average LBD: %.2f) \n", ps.distributed/std::max(1.0, double(sumLem)), average(ps.sumLbd, ps.distributed));
 		printKeyValue("Integrated", "%-6"PRIu64, ps.integrated);
 		if (accu) { printf(" (Ratio: %.2f ", ps.integrated/std::max(1.0, double(ps.distributed))); }
 		else      { printf(" ("); }
-		printf("Unit: %"PRIu64" Average Jumps: %.2f)\n", ps.imps, average(ps.jumps, ps.imps));
-		printKeyValue("Splits", "%"PRIu64"\n", (uint64)ps.splits);
+		printf("Unit: %" PRIu64" Average Jumps: %.2f)\n", ps.imps, average(ps.jumps, ps.imps));
+		printKeyValue("Splits", "%" PRIu64"\n", (uint64)ps.splits);
 		printKeyValue("Problems", "%-6"PRIu64, (uint64)ps.gps);
 		printf(" (Average Length: %.2f) \n\n", average(ps.gpLits, ps.gps));
 	}
@@ -584,29 +584,29 @@ void JsonOutput::printKey(const char* k) {
 	open_ = ",\n";
 }
 
-void JsonOutput::printValue(const char* s, const char* sep) {
-	assert(s);
-	const char* r = s;
-	while (*r && *r != '"') { ++r; }
-	if (!*r) {
-		printf("%s\"%s\"", sep, s);
-	}
-	else {
-		std::string temp(s, 0, (r - s));
-		do {
-			if (*r != '=') { temp += *r; }
-			else           { temp += "\\\""; }
-		} while (*++r);
-		printf("%s\"%s\"", sep, temp.c_str());
+void JsonOutput::printString(const char* v, const char* sep) {
+	assert(v);
+	const uint32 BUF_SIZE = 1024;
+	char buf[BUF_SIZE];
+	uint32 n = 0;
+	buf[n++] = '"';
+	while (*v) {
+		if      (*v != '\\' && *v != '"')                       { buf[n++] = *v++; }
+		else if (*v == '"' || !strchr("\"\\/\b\f\n\r\t", v[1])) { buf[n++] = '\\'; buf[n++] = *v++; }
+		else                                                    { buf[n++] = v[0]; buf[n++] = v[1]; v += 2; }
+		if (n > BUF_SIZE - 2) { buf[n] = 0; printf("%s%s", sep, buf); n = 0; sep = ""; }
 	}
+	buf[n] = 0;
+	printf("%s%s\"", sep, buf);
 }
 
 void JsonOutput::printKeyValue(const char* k, const char* v) {
-	printf("%s%-*s\"%s\": \"%s\"", open_, indent_, " ", k, v);
+	printf("%s%-*s\"%s\": ", open_, indent_, " ", k);
+	printString(v,"");
 	open_ = ",\n";
 }
 void JsonOutput::printKeyValue(const char* k, uint64 v) {
-	printf("%s%-*s\"%s\": %"PRIu64, open_, indent_, " ", k, v);
+	printf("%s%-*s\"%s\": %" PRIu64, open_, indent_, " ", k, v);
 	open_ = ",\n";
 }
 void JsonOutput::printKeyValue(const char* k, uint32 v) { return printKeyValue(k, uint64(v)); }
@@ -665,7 +665,7 @@ void JsonOutput::printModel(const Model& m, const SymbolTable& index, const Enum
 	if (index.type() == SymbolTable::map_indirect) {
 		for (SymbolTable::const_iterator it = index.begin(); it != index.end(); ++it) {
 			if (m.value(it->second.lit.var()) == trueValue(it->second.lit) && !it->second.name.empty()) {
-				printValue(it->second.name.c_str(), sep);
+				printString(it->second.name.c_str(), sep);
 				sep = ", ";
 			}	
 		}
@@ -686,7 +686,7 @@ void JsonOutput::printConsequences(const SymbolTable& index, const Enumerator&,
 	printf("%-*s", indent_, " ");
 	for (SymbolTable::const_iterator it = index.begin(); it != index.end(); ++it) {
 		if (it->second.lit.watched()) {
-			printValue(it->second.name.c_str(), sep);
+			printString(it->second.name.c_str(), sep);
 			sep = ", ";
 		}
 	}
@@ -699,7 +699,7 @@ void JsonOutput::printOptimize(const SharedMinimizeData& m) {
 	printf("%-*s", indent_, " ");
 	const char* sep = "";
 	for (uint32 i = 0, end = m.numRules(); i != end; ++i) {
-		printf("%s%"PRId64, sep, opt[i]);
+		printf("%s%" PRId64, sep, opt[i]);
 		sep = ", ";
 	}
 	endObject(type_array);
@@ -999,7 +999,7 @@ void ClaspreOutput::reportProgress(const SolveEvent& e) {
 	if (format_ == Options::features_verbose) {
 		printf("\n[Iteration %u]\n", starts_);
 		printf("%-22s: %u\n", "maxLearnt", maxL);
-		printf("%-22s: %"PRIu64"\n", "maxConflicts", maxC);
+		printf("%-22s: %" PRIu64"\n", "maxConflicts", maxC);
 		printf("%-22s: %u\n", "Constraints", s.numConstraints());
 		printf("%-22s: %u\n", "LearntConstraints", s.numLearntConstraints());
 		printf("%-22s: %u\n", "FreeVars", s.numFreeVars());
