@@ -284,8 +284,8 @@ private:
 	void printThreadStats(const SolveStats** st, std::size_t num, const RunSummary& sol);
 	void printParallelStats(const SolveStats& st);
 	void printLpStats(const PreproStats& st);
-	inline void printKey(const char* k) ;
-	inline void printValue(const char* s, const char* sep);
+	inline void printKey(const char* k);
+	inline void printString(const char* s, const char* sep);
 	inline void printKeyValue(const char* k, const char* v) ;
 	inline void printKeyValue(const char* k, uint64 v);
 	inline void printKeyValue(const char* k, uint32 v);
