@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2010-2011, Benjamin Kaufmann
+// Copyright (c) 2010-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -28,6 +28,7 @@
 #include <clasp/constraint.h>
 #include <clasp/util/left_right_sequence.h>
 #include <clasp/util/misc_types.h>
+#include <clasp/util/atomic.h>
 /*!
  * \file 
  * Contains some types shared between different solvers
@@ -61,7 +62,7 @@ public:
 		uint32 litsRemoved;
 	} stats;
 protected:
-	void reportProgress(char t, uint32 min, uint32 max);
+	void reportProgress(const PreprocessEvent& ev);
 	SharedContext*  ctx_;
 private:
 	SatPreprocessor(const SatPreprocessor&);
@@ -89,6 +90,8 @@ struct ProblemStats {
 	uint32  constraints;
 	uint32  constraints_binary;
 	uint32  constraints_ternary;
+	uint32  size;
+	uint32  complexity;
 	void    reset() { std::memset(this, 0, sizeof(*this)); }
 	void diff(const ProblemStats& o) {
 		vars               = std::max(vars, o.vars)-std::min(vars, o.vars);
@@ -179,12 +182,62 @@ public:
 	uint32 numTernary()const { return tern_[0]; }
 	uint32 numLearnt() const { return bin_[1] + tern_[1]; }
 	uint32 numEdges(Literal p) const;
+	//! Applies op on all unary- and binary implications following from p.
+	/*!
+	 * OP must provide two functions:
+	 *  - bool unary(Literal, Literal)
+	 *  - bool binary(Literal, Literal, Literal)
+	 * The first argument will be p, the second (resp. third) the unary
+	 * (resp. binary) clause implied by p.
+	 * \note For learnt imps, at least one literal has its watch-flag set.
+	 */
+	template <class OP>
+	bool forEach(Literal p, const OP& op) const {
+		const ImplicationList& x = graph_[p.index()];
+		if (x.empty()) return true;
+		ImplicationList::const_right_iterator rEnd = x.right_end(); // prefetch
+		for (ImplicationList::const_left_iterator it = x.left_begin(), end = x.left_end(); it != end; ++it) {
+			if (!op.unary(p, *it)) { return false; }
+		}
+		for (ImplicationList::const_right_iterator it = x.right_begin(); it != rEnd; ++it) {
+			if (!op.binary(p, it->first, it->second)) { return false; }
+		}
+#if WITH_THREADS
+		for (Block* b = (x).learnt; b ; b = b->next) {
+			p.watch(); bool r = true;
+			for (Block::const_iterator imp = b->begin(), endOf = b->end(); imp != endOf; ) {
+				if (!imp->watched()) { r = op.binary(p, imp[0], imp[1]); imp += 2; }
+				else                 { r = op.unary(p, imp[0]);          imp += 1; }
+				if (!r)              { return false; }
+			}
+		}
+#endif
+		return true;
+	}
 private:
 	ShortImplicationsGraph(const ShortImplicationsGraph&);
 	ShortImplicationsGraph& operator=(ShortImplicationsGraph&);
-#ifndef DISABLE_MULTI_THREADING
-	struct Block;
-	typedef std::atomic<Block*> SharedBlockPtr;
+	struct Propagate;
+	struct ReverseArc;
+#if WITH_THREADS
+	struct Block {
+		typedef std::atomic<uint32> atomic_size;
+		typedef std::atomic<Block*> atomic_ptr;
+		typedef const Literal*      const_iterator;
+		typedef       Literal*      iterator;
+		enum { block_cap = (64 - (sizeof(atomic_size)+sizeof(atomic_ptr)))/sizeof(Literal) };
+		explicit Block();
+		const_iterator  begin() const { return data; }
+		const_iterator  end()   const { return data+size(); }
+		iterator        end()         { return data+size(); }
+		uint32          size()  const { return size_lock >> 1; }
+		bool tryLock(uint32& lockedSize);
+		void addUnlock(uint32 lockedSize, const Literal* x, uint32 xs);
+		atomic_ptr  next;
+		atomic_size size_lock;
+		Literal     data[block_cap];
+	};
+	typedef Block::atomic_ptr SharedBlockPtr;
 	typedef bk_lib::left_right_sequence<Literal, std::pair<Literal,Literal>, 64-sizeof(SharedBlockPtr)> ImpListBase;
 	struct ImplicationList : public ImpListBase {
 		ImplicationList() : ImpListBase() { learnt = 0; }
@@ -213,21 +266,18 @@ private:
 //! Base class for distributing learnt knowledge between solvers.
 class Distributor {
 public:
-	Distributor(uint32 maxShare, uint32 typesToShare, uint32 maxLbd);
+	static  uint64  mask(uint32 i)             { return uint64(1) << i; }
+	static  uint32  initSet(uint32 sz)         { return (uint64(1) << sz) - 1; }
+	static  bool    inSet(uint64 s, uint32 id) { return (s & mask(id)) != 0; }
+	Distributor();
 	virtual ~Distributor();
-	SharedLiterals* publish(const Solver& source, const Literal* lits, uint32 size, const ClauseInfo& extra);
+	virtual void    publish(const Solver& source, SharedLiterals* lits) = 0;
 	virtual uint32  receive(const Solver& in, SharedLiterals** out, uint32 maxOut) = 0;
-protected:
-	virtual void    doPublish(const Solver& source, SharedLiterals* lits) = 0;
 private:
 	Distributor(const Distributor&);
 	Distributor& operator=(const Distributor&);
-	uint32 maxShare_;
-	uint32 lbdMax_    : 16;
-	uint32 typeMask_  : 16;
 };
 
-	
 //! Aggregates information to be shared between solver objects.
 /*!
  * Among other things, SharedContext objects store 
@@ -247,87 +297,123 @@ public:
 	typedef ProblemStats                   Stats;
 	typedef LitVec::size_type              size_type;
 	typedef ShortImplicationsGraph         BTIG;
+	typedef Distributor*                   DistrPtr;
 	
-	enum InitMode { init_share_symbols };
-	
+	enum InitMode        { init_share_symbols };
+	enum PhysicalSharing { share_no   = 0, share_problem = 1, share_learnt = 2, share_all = 3 };
+	enum UpdateMode      { update_propagate = 0, update_conflict = 1 };
+
+	/*!
+	 * \name configuration
+	 */
+	//@{
 	//! Creates a new object for sharing variables and the binary and ternary implication graph.
-	SharedContext();
-	//! Creates a new object that shares its symbol table with rhs
+	SharedContext(PhysicalSharing x = share_no, bool learnImp = true);
+	//! Creates a new object that shares its symbol table with rhs.
 	SharedContext(const SharedContext& rhs,  InitMode m);
-	
 	~SharedContext();
-	
-	//! Enables sharing of initial problem constraints.
-	/*!
-	 * If this function is not called, problem constraints are
-	 * cloned whenever a solver attaches to this object. Otherwise,
-	 * they are shared.
-	 */
-	void enableConstraintSharing() {  shareConstr_ = true; }
-
-	//! Enables sharing of learnt constraints.
-	/*!
-	 * If this function is not called, learnt constraints are
-	 * not shared between different solvers. Otherwise,
-	 * sharing is possible and controlled by a distribution 
-	 * strategy.
-	 */
-	void enableLearntSharing(Distributor* d) { 
-		distributor_.reset(d); 
-	}
-
-	//! Enable/disable learning of binary/ternary clauses via ShortImplicationsGraph.
-	void enableUpdateShortImplications(bool b) {
-		updateShort_ = b;
-	}
-	//! Returns whether dynamic updates of ShortImplicationsGraph are allowed. 
-	bool updateShortImplications() const { return !frozen_ || updateShort_; }
-
+	//! Enables progress reporting via the given report callback.
+	void       enableProgressReport(ProgressReport* r) { progress_ = r; }
 	//! Resets this object to the state after default construction.
-	void reset();
+	void       reset();
+	//! Sets maximal number of solvers sharing this object.
+	void       setSolvers(uint32 numSolver);
+	//! Configures physical sharing of (explicit) constraints.
+	/*!
+	 * If x is share_no, explicit constraints are not shared but copied.
+	 * If x is share_problem, only problem constraints are shared while learnt constraints are copied.
+	 * If x is share_learnt, learnt constraints are shared while problem constraints are copied.
+	 * If x is share_all, all constraints are shared.
+	 */
+	void       physicalSharing(PhysicalSharing x)   { share_.physical = x; }
+	//! Enables/disables learning of small clauses via ShortImplicationsGraph.
+	void       learnImplicit(bool b)                { share_.impl = (uint32)b; }
+	//! Set mt update mode.
+	void       updateMode(UpdateMode x)             { share_.update = x;       }
+	//! Sets the global distribution strategey to use.
+	/*!
+	 * \note The caller also has to set a distributor in order to enable
+	 *       distribution of learnt constraints.
+	 */
+	void       setDistribution(uint32 maxSize, uint32 maxLbd, uint32 types) {
+		share_.distSize = maxSize;
+		share_.distLbd  = maxLbd;
+		share_.distMask = types;
+	}
+	//! Sets the distributor object to use for distribution of learnt constraints.
+	/*!
+	 * If this function is not called or d is 0, (explicit) learnt constraints are
+	 * never distributed to other solvers. Otherwise,
+	 * distribution is controlled by the global distribution strategy.
+	 * \see setDistribution(uint32 maxSize, uint32 maxLbd, uint32 types)
+	 */
+	void       setDistributor(DistrPtr d) { distributor_.reset(d); }
+	DistrPtr   releaseDistributor()       { return distributor_.release(); }
+	
+	
+	uint32     numSolvers() const { return share_.count; }
+	bool       frozen()     const { return share_.frozen; }
+	bool       isShared()   const { return frozen() && numSolvers() > 1; }
+	UpdateMode updateMode() const { return UpdateMode(share_.update); }
+	bool       physicalShare(ConstraintType t) const{ return (share_.physical & (1 + (t != Constraint_t::static_constraint))) != 0; }
+	bool       physicalShareProblem()          const{ return (share_.physical & share_problem) != 0; }
+	bool       distribution()                  const{ return share_.distMask != 0; }
+	bool       allowImplicit(ConstraintType t) const{ return t != Constraint_t::static_constraint ? share_.impl != 0 : !isShared(); }
+	//@}
 	
-	//! Returns true if var represents a valid variable in this object.
 	/*!
-	 * \note The range of valid variables is [1..numVars()]. The variable 0
-	 * is a special sentinel variable. 
+	 * \name problem specification
+	 * Functions for adding a problem to the master solver.
+	 * Problem specification is a four-stage process:
+	 * -# Add variables to the SharedContext object.
+	 * -# Call startAddConstraints().
+	 * -# Add problem constraints to the master solver.
+	 * -# Call endInit() to finish the initialization process.
+	 * .
+	 * \note After endInit() was called, other solvers can be attached to this object.
+	 * \note In incremental setting, the process must be repeated for each incremental step.
+	 * 
+	 * \note Problem specification is *not* thread-safe, i.e. during initialization no other thread shall
+	 * access the context.
+	 *
+	 * \note !frozen() is a precondition for all functions in this group!
 	 */
-	bool validVar(Var var) const { return var <= numVars(); }
-
-	//! Returns the number of problem variables.
-	/*!
-	 * \note The special sentinel-var 0 is not counted, i.e. numVars() returns
-	 * the number of problem-variables.
-	 * To iterate over all problem variables use a loop like:
-	 * \code
-	 * for (Var i = 1; i <= numVars(); ++i) {...}
-	 * \endcode
-	 */
-	uint32 numVars() const { return varInfo_.numVars() - 1; }
-
-	//! Returns the number of eliminated vars.
-	uint32 numEliminatedVars()const { return problem_.vars_eliminated; }
-
+	//@{
+	
 	//! Reserves space for at least varGuess variables.
 	/*!
-	 * \param varGuess number of vars to reserve space for
+	 * \param varGuess Number of vars to reserve space for.
 	 * \note If the number of variables is known upfront, passing the correct value
 	 * for varGuess avoids repeated regrowing of the state data structures.
 	 */
-	void reserveVars(uint32 varGuess);
+	void    reserveVars(uint32 varGuess);
 
 	//! Adds a new variable of type t.
 	/*!
-	 * \param t  type of the new variable (either Var_t::atom_var or Var_t::body_var)
-	 * \param eq true if var represents both an atom and a body. In that case
+	 * \param t  Type of the new variable (either Var_t::atom_var or Var_t::body_var).
+	 * \param eq True if var represents both an atom and a body. In that case
 	 *           t is the variable's primary type and determines the preferred literal.
-	 * \return The index of the new variable
-	 * \note The problem variables are numbered from 1 onwards!
+	 * \return The index of the new variable.
+	 * \note Problem variables are numbered from 1 onwards!
 	 */
-	Var addVar(VarType t, bool eq = false);
-
+	Var     addVar(VarType t, bool eq = false);
+	//! Request additional reason data slot for variable v.
+	void    requestData(Var v);
+	//! Freezes/defreezes a variable (a frozen var is exempt from SatELite preprocessing).
+	void    setFrozen(Var v, bool b);
+	//! Adds v to resp. removes v from the set of projection variables.
+	void    setProject(Var v, bool b)    { assert(validVar(v)); if (b != varInfo_.isSet(v, VarInfo::PROJECT)) varInfo_.toggle(v, VarInfo::PROJECT); }
+	//! Marks/unmarks v as contained in a negative loop or head of a choice rule.
+	void    setNant(Var v, bool b)       { assert(validVar(v)); if (b != varInfo_.isSet(v, VarInfo::NANT))    varInfo_.toggle(v, VarInfo::NANT);    }
+	//! Eliminates the variable v.
+	/*!
+	 * \pre v must not occur in any constraint and frozen(v) == false and value(v) == value_free
+	 */
+	void    eliminate(Var v);
+	
 	//! Requests a special tag literal for tagging conditional knowledge.
 	/*!
-	 * Once a tag literal p is set, newly learnt clauses containing ~p are
+	 * Once a tag literal p is set, learnt clauses containing ~p are
 	 * tagged as "conditional". Conditional clauses can be removed from a solver
 	 * by calling Solver::removeConditional(). Furthermore, calling 
 	 * Solver::strengthenConditional() removes ~p from conditional clauses and
@@ -337,24 +423,47 @@ public:
 	 *       the whole search. 
 	 */
 	void    requestTagLiteral();
-	Literal tagLiteral() const   { return tag_; }
 	void    removeTagLiteral();
-
-	//! Request additional reason data slot for variable v.
-	void    requestData(Var v);
-
+	
+	//! Returns the number of problem variables.
+	/*!
+	 * \note The special sentinel-var 0 is not counted, i.e. numVars() returns
+	 * the number of problem-variables.
+	 * To iterate over all problem variables use a loop like:
+	 * \code
+	 * for (Var i = 1; i <= numVars(); ++i) {...}
+	 * \endcode
+	 */
+	uint32  numVars()          const { return varInfo_.numVars() - 1; }
+	//! Returns the number of eliminated vars.
+	uint32  numEliminatedVars()const { return problem_.vars_eliminated; }
+	
+	//! Returns true if var represents a valid variable in this object.
+	/*!
+	 * \note The range of valid variables is [1..numVars()]. The variable 0
+	 * is a special sentinel variable. 
+	 */
+	bool    validVar(Var var)  const { return var <= numVars(); }
 	//! Returns the type of variable v.
 	/*!
 	 * If v was added with parameter eq=true, the return value
 	 * is Var_t::atom_body_var.
 	 */
-	VarType type(Var v) const {
+	VarType type(Var v)        const {
 		assert(validVar(v));
 		return varInfo_.isSet(v, VarInfo::EQ)
 			? Var_t::atom_body_var
 			: VarType(Var_t::atom_var + varInfo_.isSet(v, VarInfo::BODY));
 	}
-
+	//! Returns true if v is currently eliminated, i.e. no longer part of the problem.
+	bool    eliminated(Var v)  const { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::ELIM); }
+	//! Returns true if v is excluded from variable elimination.
+	bool    frozen(Var v)      const { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::FROZEN); }
+	//! Returns true if v is a projection variable.
+	bool    project(Var v)     const { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::PROJECT);}
+	//! Returns true if v is contained in a negative loop or head of a choice rule.
+	bool    nant(Var v)        const { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::NANT);}
+	Literal tagLiteral()       const { return tag_; }
 	//! Returns the preferred decision literal of variable v w.r.t its type.
 	/*!
 	 * \return 
@@ -367,135 +476,74 @@ public:
 		assert(validVar(v));
 		return Literal(v, !varInfo_.isSet(v, VarInfo::BODY));
 	}
-
-	//! Returns true if v is currently eliminated, i.e. no longer part of the problem.
-	bool eliminated(Var v)  const     { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::ELIM); }
-	//! Returns true if v is excluded from variable elimination.
-	bool frozen(Var v)      const     { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::FROZEN); }
-	//! Returns true if v is a projection variable.
-	bool project(Var v)     const     { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::PROJECT);}
-	//! Returns true if v is contained in a negative loop or head of a choice rule.
-	bool nant(Var v)        const     { assert(validVar(v)); return varInfo_.isSet(v, VarInfo::NANT);}
 	
-	//! Freezes/defreezes a variable (a frozen var is exempt from SatELite preprocessing).
-	void setFrozen(Var v, bool b);
-	void setProject(Var v, bool b)    { assert(validVar(v)); if (b != varInfo_.isSet(v, VarInfo::PROJECT)) varInfo_.toggle(v, VarInfo::PROJECT); }
-	void setNant(Var v, bool b)       { assert(validVar(v)); if (b != varInfo_.isSet(v, VarInfo::NANT))    varInfo_.toggle(v, VarInfo::NANT);    }
+	//! Prepares master solver so that constraints can be added.
 	/*!
-	 * \name problem specification
-	 * Functions for adding a problem to the master solver.
-	 * Problem specification is a four-stage process:
-	 * - first, add variables to the SharedContext object 
-	 * - second, call startAddConstraints()
-	 * - third, add problem constraints
-	 * - finally, endInit() shall be called to finished the initialization process
-	 * .
-	 * \note After endInit() was called, other solvers can be attached to this object
-	 * \note In incremental setting, the process must be repeated for each incremental step.
-	 * 
-	 * \note Problem specification is *not* thread-safe, i.e. during initialization no other thread shall
-	 * access the context.
-	 *
-	 * \note unique() is a precondition for all functions in this group!
-	 */
-	//@{
-
-	bool unique() const   { return shareCount_ == 1; } 
-
-	//! Starts an initialization phase.
-	/*!
-	 * Must be called to publish variables and before constraints over
-	 * previously added variables can be added.
+	 * Must be called to publish previously added variables to master solver
+	 * and before constraints over these variables can be added.
+	 * \post !frozen()
+	 * \return The master solver associated with this object.
 	 */
 	Solver& startAddConstraints(uint32 constraintGuess = 100);
-
-	//! Returns the number of problem constraints.
-	uint32 numConstraints()   const;
-
-	//! Eliminates the variable v.
-	/*!
-	 * \pre v must not occur in any constraint 
-	 *  and frozen(v) == false and value(v) == value_free
-	 */
-	void eliminate(Var v);
-
-	//! Adds the constraint c to the master solver
-	/*!
-	 * \pre endInit() was not called.
-	 */
-	void add(Constraint* c);
-
-	//! Adds the unary constraint p to the master solver.
-	/*!
-	 * \note Unary constraints are immediately asserted.
-	 * \return false if asserting p leads to a conflict.
-	 */
-	bool addUnary(Literal p);
-
-	//! Adds the binary constraint (p, q) to the shared implication graph.
-	void addBinary(Literal p, Literal q) { btig_.addBinary(p, q, false); }
-	
-	//! Adds the ternary constraint (p, q, r) to the shared implication graph.
-	void addTernary(Literal p, Literal q, Literal r) { btig_.addTernary(p, q, r, false); }
-
-	//! Adds p as post propagator to the master solver.
-	/*!
-	 * \pre p was not added previously and is not part of any other solver
-	 * \note Post propagators are stored in priority order.
-	 * \see PostPropagator::priority()
-	 */
-	void addPost(PostPropagator* p);
-
 	//! Attaches the given enumerator to this object.
 	/*!
 	 * \note ownership is transferred
 	 * \note In incremental setting, the enumerator must be reattached in
 	 *       each incremental step by calling addEnumerator(enumerator());
 	 */
-	void        addEnumerator(Enumerator* en);
-	Enumerator* enumerator() const { return enumerator_.get(); }
-
+	void    addEnumerator(Enumerator* en);
+	//! Same as master()->addUnary(p, Constraint_t::static_constraint)
+	bool    addUnary(Literal p);
+	//! Same as master()->add(c)
+	void    add(Constraint* c);
+	//! Same as shortImplications->addBinary(p, q, learnt).
+	bool    addBinary(Literal p, Literal q, bool learnt = false)             { return btig_.addBinary(p, q, learnt); }
+	//! Same as shortImplications->addTernary(p, q, learnt).
+	bool    addTernary(Literal p, Literal q, Literal r, bool learnt = false) { return btig_.addTernary(p, q, r, learnt); }
+	
 	//! Finishes initialization of the master solver.
 	/*!
-	 * endInit must be called once before search is started. After endInit()
-	 * was called, a number (shareCount-1) of other solvers can be attached to the 
+	 * The function must be called once before search is started. After endInit()
+	 * was called, numSolvers()-1 other solvers can be attached to the 
 	 * shared context and learnt constraints may be added to solver.
-	 * \return 
-	 *  - false if the constraints are initially conflicting. True otherwise.
+	 * \return If the constraints are initially conflicting, false. Otherwise, true.
 	 * \note
 	 * The master solver can't recover from top-level conflicts, i.e. if endInit()
 	 * returned false, the solver is in an unusable state.
+	 * \post frozen()
 	 */
-	bool endInit(uint32 shareCount = 1);
-
+	bool    endInit();
+	
 	//! Attaches s to this object.
 	/*!
-	 * \pre other != master()
+	 * \pre other is not already attached to some other shared context.
 	 * \note It is safe to attach multiple solvers concurrently
 	 * but the master solver shall not change during the whole
 	 * operation.
 	 */
-	bool attach(Solver& other);
-
+	bool    attach(Solver& other);
+	
 	//! Detaches s from this object.
 	/*!
-	 * The function removes any enumeration related constraints from s.
+	 * The function removes any tentative constraints from s.
 	 * Shall be called once after search has stopped.
 	 * \note The function is concurrency-safe w.r.t to different solver objects, 
 	 *       i.e. in a parallel search different solvers may call detach()
 	 *       concurrently.
 	 */
-	void detach(Solver& s);
+	void    detach(Solver& s);
 
+	//! Returns the number of problem constraints.
+	uint32      numConstraints()    const;
 	//! Estimates the problem complexity.
 	/*!
 	 * \return sum of c->estimateComplexity(*master()) for each problem 
 	 *         constraint c.
 	 */
-	uint32 problemComplexity() const;
-
+	uint32      problemComplexity() const;
 	//! Returns the size of top-level after last call to endInit()
-	size_type topLevelSize() const { return lastTopLevel_; }
+	size_type   topLevelSize()      const { return lastTopLevel_; }
+	Enumerator* enumerator()        const { return enumerator_.get(); }
 	//@}
 
 	/*!
@@ -506,26 +554,7 @@ public:
 	 * from multiple threads.
 	 */
 	//@{
-
-	//! Learns the unary clause (p) and distributes it to attached threads.
-	/*! 
-	 * \pre s.isTrue(p)
-	 */
-	bool learnUnary(Solver& s, Literal p, ConstraintType t);
-
-	//! Learns the binary clause (p, q) and distributes it to attached threads.
-	/*!
-	 * \return true if the clause was added.
-	 */
-	bool learnBinary(Solver& s, Literal p, Literal q, const ClauseInfo& info);
-	
-	//! Learns the ternary clause (p, q, r) and distributes it to attached threads.
-	/*!
-	 * \return true if the clause was added.
-	 */
-	bool learnTernary(Solver& s, Literal p, Literal q, Literal r, const ClauseInfo& info);
-	
-	//! Distributes the clause in lits to other threads.
+	//! Distributes the clause in lits via the distributor.
 	/*!
 	 * The function first calls the distribution strategy 
 	 * to decides whether the clause is a good candidate for distribution.
@@ -541,36 +570,41 @@ public:
 	 *   responsibility to release the returned handle (i.e. by calling release()).
 	 */
 	SharedLiterals* distribute(const Solver& owner, const Literal* lits, uint32 size, const ClauseInfo& extra) const;
-	void            distribute(const Solver& owner, Literal p, Literal q, Literal r, ConstraintType type) const;
-
 	//! Tries to receive at most maxOut clauses.
 	/*!
 	 * The function queries the distribution object for new clauses to be delivered to
 	 * the solver target. Clauses are stored in out.
-	 * \return the number of clauses received
+	 * \return The number of clauses received.
 	 */
-	uint32 receive(const Solver& target, SharedLiterals** out, uint32 maxOut) const;
-
+	uint32          receive(const Solver& target, SharedLiterals** out, uint32 maxOut) const;
 	//! Returns the number of learnt binary and ternary clauses
-	uint32       numLearntShort() const { return btig_.numLearnt(); }
+	uint32          numLearntShort() const { return btig_.numLearnt(); }
 
 	//@}
-	//! Returns the master solver associated with.
+	//! Returns the master solver associated with this object.
 	Solver*      master()    const   { return master_; }	
-	bool         shareConstraints() const { return shareConstr_; }
 	const Stats& stats()     const   { return problem_; }
+	uint32       winner()    const   { return share_.winner; }
+	void         setWinner(uint32 x) { share_.winner = std::min(x, numSolvers()); }
 	uint32       numBinary() const   { return btig_.numBinary();  }
 	uint32       numTernary()const   { return btig_.numTernary(); }
 	SymbolTable& symTab()    const   { return symTabPtr_->symTab; }
-	void         simplifyBtig(Literal p) { btig_.removeTrue(*master(), p); }
+	BTIG&        shortImplications() { return btig_; }
 	const BTIG&  shortImplications() const { return btig_; }
 	uint32       numShortImplications(Literal p) const { return btig_.numEdges(p); }	
+	void         reportProgress(const PreprocessEvent& ev) const { if (progress_) progress_->reportProgress(ev);  }
+	void         reportProgress(const SolveEvent& ev)      const { if (progress_) progress_->reportProgress(ev);  }
+	void         setProblemSize(uint32 sz, uint32 estimate) {
+		problem_.size      = sz;
+		problem_.complexity= estimate;
+	}
 	SatPrepro    satPrepro;  // preprocessor
 private:
 	SharedContext(const SharedContext&);
 	SharedContext& operator=(const SharedContext&);
 	typedef std::auto_ptr<Distributor> DistPtr;
 	typedef std::auto_ptr<Enumerator>  EnumPtr;
+	typedef ProgressReport*            LogPtr;
 	struct SharedSymTab {
 		SharedSymTab() : refs(1) {}
 		SymbolTable symTab;
@@ -580,15 +614,26 @@ private:
 	VarInfo      varInfo_;     // varInfo[v] stores info about variable v
 	BTIG         btig_;        // binary-/ternary implication graph
 	Solver*      master_;      // master solver, responsible for init
+	LogPtr       progress_;    // report interface or 0 if not used
 	EnumPtr      enumerator_;  // enumerator object
 	DistPtr      distributor_; // object for distributing learnt knowledge
 	Literal      tag_;         // literal for tagging learnt constraints
 	size_type    lastInit_;    // size of master's db after last init
 	size_type    lastTopLevel_;// size of master's top-level after last init
-	uint32       shareCount_;  // number of objects sharing this object
-	bool         shareConstr_; // sharing of problem constraints enabled?
-	bool         updateShort_; // allow update of BTIG after endInit()?
-	bool         frozen_;
+	struct Share {             // Share-Flags
+		uint32 count   :14;      //   max number of objects sharing this object
+		uint32 winner  :14;      //   id of solver that terminated the search
+		uint32 physical: 2;      //   mode of physical sharing 
+		uint32 impl    : 1;      //   allow update of BTIG after endInit()?
+		uint32 frozen  : 1;      //   is adding of problem constraints allowed?
+		uint32 distSize: 20;     //   distribute constraints up to this size only
+		uint32 distLbd : 8;      //   distribute constraints up to this lbd only
+		uint32 distMask: 3;      //   distribute constraints of this type only
+		uint32 update  : 1;      //   process update messages on propagation or after conflict
+		bool   distribute(uint32 size, uint32 lbd, uint32 type) const {
+			return size <= distSize && lbd <= distLbd && ((type & distMask) != 0);
+		}
+	}            share_;
 };
 
 //@}
