@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -86,13 +86,14 @@ struct SearchLimits {
 class Solver {
 	friend class SharedContext;
 	void startInit(SharedContext* ctx, uint32 constraintGuess);
+	bool endInit();
 	void setEnumerationConstraint(Constraint* c) {
 		if (enum_) enum_->destroy(this, true);
 		enum_ = c;
 	}
-	bool endInit();
 public:
 	typedef PodVector<Constraint*>::type      ConstraintDB;
+	typedef std::auto_ptr<DecisionHeuristic>  Heuristic;
 	/*!
 	 * \name Construction/Destruction/Setup
 	 */
@@ -109,6 +110,8 @@ public:
 	 */
 	void   setId(uint32 id) { id_ = id; }
 	uint32 id() const       { return id_; }
+	//! Used to signal an error in this solver.
+	static const   uint32 invalidId = UINT32_MAX;
 	
 	//! Resets a solver object to the state it had after default-construction.
 	void reset();
@@ -116,53 +119,65 @@ public:
 	//! Shuffle constraints upon next simplification.
 	void shuffleOnNextSimplify() { shuffle_ = true; }
 	
+	//! Sets the decision heuristic to use in this object.
+	void setHeuristic(uint32 heuId, DecisionHeuristic* h) {
+		strategies().heuId = heuId;
+		heuristic_.reset(h);
+	}
+	
 	//! Returns the strategies used by this solver-object. 
 	SolverStrategies& strategies() { return strategy_; }
+	Heuristic&        heuristic()  { return heuristic_;}
 
 	/*!
 	 * \overload SolverStrategies& Solver::strategies()
 	 */
 	const SolverStrategies& strategies() const { return strategy_; }
+	const Heuristic&        heuristic()  const { return heuristic_;}
 
 	//! Returns a pointer to the shared context object of this solver.
 	const SharedContext* sharedContext() const { return shared_; }
 
 	//! Returns a pointer to the sat-preprocessor used by this solver.
 	SatPreprocessor* satPrepro() const;
-
-	//@}	
-
-	/*!
-	 * \name Static state inspection
-	 * Functions for inspecting the static state of the solver.
-	 * \note validVar(v) is a precondition for all functions that take a variable as 
-	 * parameter.
-	 */
-	//@{
 	
-	//! Returns the number of problem variables.
+	//! Adds the unary constraint p to the solver.
 	/*!
-	 * \note The special sentine-var 0 is not counted, i.e. numVars() returns
-	 * the number of problem-variables.
-	 * To iterate over all problem variables use a loop like:
-	 * \code
-	 * for (Var i = 1; i <= s.numVars(); ++i) {...}
-	 * \endcode
+	 * \note Unary constraints are immediately asserted.
+	 * \return false if asserting p leads to a conflict.
 	 */
-	uint32 numVars() const { return assign_.numVars() - 1; }
+	bool addUnary(Literal p, ConstraintType);	
+	//! Adds the short clause cl to the solver.
+	/*! 
+	 * \pre  cl is either binary or ternary (sz > 1 && sz < 4)
+	 * \note cl is stored in the short implication graph unless
+	 *       updates of this graph are not permitted.
+	 * \see  SharedContext::allowImplicit(ConstraintType)
+	 */
+	void addShort(const Literal* cl, uint32 sz, const ClauseInfo& extra);
+	//! Adds the binary clause [p q] by calling addShort.
+	void addBinary(Literal p, Literal q, const ClauseInfo& extra) { Literal x[2] = {p,q}; addShort(x, 2, extra); }
+	//! Adds the ternary clause [p q r] by calling addShort.
+	void addTernary(Literal p, Literal q, Literal r, const ClauseInfo& extra) { Literal x[3] = {p,q,r}; addShort(x, 3, extra); }
+	
+	//! Adds the problem constraint c to the solver.
+	void add(Constraint* c);
 
-	//! Returns true if var represents a valid variable in this solver.
+	//! Adds p as post propagator to this solver.
 	/*!
-	 * \note The range of valid variables is [1..numVars()]. The variable 0
-	 * is a special sentinel variable that is always true. 
+	 * \pre p was not added previously and is not part of any other solver.
+	 * \note Post propagators are stored and called in priority order.
+	 * \see PostPropagator::priority()
 	 */
-	bool validVar(Var var)     const { return var <= numVars(); }
-	bool validLevel(uint32 dl) const { return dl != 0 && dl <= decisionLevel(); }
+	void addPost(PostPropagator* p)    { post_.add(p); }
 
-	//! Number of problem constraints in this solver.
-	uint32 numConstraints() const;
-
-	//@}
+	//! Removes p from the solver's list of post propagators.
+	/*!
+	 * \note removePost(p) shall only be called during propagation
+	 *       of p or if no propagation is currently active.
+	 */
+	void removePost(PostPropagator* p) { post_.remove(p); }
+	
 	/*!
 	 * \name CDNL-functions.
 	 */
@@ -230,13 +245,13 @@ public:
 
 	//! Sets the backtracking level to dl.
 	void setBacktrackLevel(uint32 dl) {
-		btLevel_    = std::max(std::min(dl, decisionLevel()), rootLevel_);
+		btLevel_ = std::max(std::min(dl, decisionLevel()), rootLevel_);
 	}
 	//! Returns the current backtracking level.
 	uint32 backtrackLevel() const { return btLevel_; }
 
 	//! Returns whether the solver can split-off work.
-	bool   splittable() const { return decisionLevel() > rootLevel() && !frozenLevel(rootLevel()+1); }
+	bool splittable() const { return decisionLevel() > rootLevel() && !frozenLevel(rootLevel()+1); }
 
 	//! Copies the solver's currrent guiding path to gp.
 	/*!
@@ -249,7 +264,7 @@ public:
 	 * \param[in,out] startPos    position in the trail from which copying should start
 	 * \param[in,out] numImplied  number of newly implied literals in gp
 	 */
-	void   updateGuidingPath(LitVec& gp, LitVec::size_type& startPos, uint32& numImplied);
+	void updateGuidingPath(LitVec& gp, LitVec::size_type& startPos, uint32& numImplied);
 
 	//! If called on top-level, removes SAT-clauses + Constraints for which Constraint::simplify returned true.
 	/*!
@@ -419,6 +434,16 @@ public:
 	 */
 	uint32 inDegree(WeightLitVec& out);
 	
+	//! Adds the learnt constraint c to the solver.
+	/*!
+	 * \pre endInit() was called.
+	 */
+	void addLearnt(LearntConstraint* c, uint32 size, ConstraintType type) {
+		learnts_.push_back(c);
+		stats.addLearnt(size, type); 
+	}
+	void addLearnt(LearntConstraint* c, uint32 size) { addLearnt(c, size, c->type()); }
+	
 	//! Removes upto remMax percent of the learnt nogoods.
 	/*!
 	 * \param remMax  Fraction of nogoods to remove ([0.0,1.0]).
@@ -463,7 +488,7 @@ public:
 	/*!
 	 * Similar to undoUntil but optionally also pops the backtrack-level
 	 * to dl if possible.
-	 * \return Zhe current decision level.
+	 * \return The current decision level.
 	 */
 	uint32 undoUntil(uint32 dl, bool popBtLevel);
 
@@ -475,215 +500,110 @@ public:
 	 * \pre the current assignment is a model
 	 */ 
 	bool nextSymModel();
-
 	//@}  
-
-	/*!
-	 * \name Learning functions
-	 * Functions controlling learning.
-	 */
-	//@{
-		
-	//! Wraps SharedContext::add() - only provided for convenience.
-	void add(Constraint* c);
-
-	//! Adds the unary constraint p to the solver.
-	/*!
-	 * \note Unary constraints are immediately asserted.
-	 * \return false if asserting p leads to a conflict.
-	 */
-	bool addUnary(Literal p, ConstraintType);
-
-	//! Wraps SharedContext::addBinary() resp. SharedContext::learnBinary() depending on the type t.
-	bool addBinary(Literal p, Literal q, const ClauseInfo& t) {
-		t.type() == Constraint_t::static_constraint
-			? shared_->addBinary(p,q)
-			: (void)shared_->learnBinary(*this, p, q, t);
-		return true;
-	}
 	
-	//! Wraps SharedContext::addTernary() resp. SharedContext::learnTernary() depending on the type t.
-	bool addTernary(Literal p, Literal q, Literal r, const ClauseInfo& t) {
-		t.type() == Constraint_t::static_constraint
-			? shared_->addTernary(p,q, r)
-			: (void)shared_->learnTernary(*this, p, q, r, t);
-		return true;
-	}
-	
-	//! Adds the learnt constraint c to the solver.
 	/*!
-	 * \pre endInit() was called.
-	 */
-	void addLearnt(LearntConstraint* c, uint32 size, bool tagged = false) {
-		learnts_.push_back(c);
-		stats.addLearnt(size, c->type()); 
-		tagged_ += tagged;
-	}
-	
-	//! Adds p as post propagator to this solver.
-	/*!
-	 * \pre p was not added previously and is not part of any other solver.
-	 * \note Post propagators are stored and called in priority order.
-	 * \see PostPropagator::priority()
-	 */
-	void addPost(PostPropagator* p)    { post_.add(p); }
-
-	//! Removes p from the solver's list of post propagators.
-	/*!
-	 * \note removePost(p) shall only be called during propagation
-	 *       of p or if no propagation is currently active.
-	 */
-	void removePost(PostPropagator* p) { post_.remove(p); }
-
-	//! Returns the idx'th learnt constraint.
-	/*!
-	 * \pre idx < numLearntConstraints()
-	 */
-	LearntConstraint& getLearnt(uint32 idx) const {
-		assert(idx < numLearntConstraints());
-		return *static_cast<LearntConstraint*>(learnts_[ idx ]);
-	}
-
-	//@}
-
-	/*!
-	 * \name Dynamic state inspection
-	 * Functions for inspecting the state of the search.
+	 * \name state inspection
+	 * Functions for inspecting the state of the solver & search.
 	 * \note validVar(v) is a precondition for all functions that take a variable as 
 	 * parameter.
 	 */
 	//@{
-
-	//! Returns the number of assigned variables.
-	/*!
-	 * \note The special sentinel-var 0 is not counted.
-	 */
-	uint32 numAssignedVars()  const { return assign_.assigned(); }
 	
+	//! Returns the number of problem variables.
+	/*!
+	 * \see SharedContext::numVars()
+	 */
+	uint32   numVars()              const { return assign_.numVars() - 1; }
+	//! Returns true if var represents a valid variable in this solver.
+	/*!
+	 * \see SharedContext::validVar(Var v)
+	 */
+	bool     validVar(Var var)      const { return var <= numVars(); }
+	//! Returns the number of assigned problem variables.
+	uint32   numAssignedVars()      const { return assign_.assigned(); }
 	//! Returns the number of free variables.
 	/*!
 	 * The number of free variables is the number of vars that are neither
 	 * assigned nor eliminated.
 	 */
-	uint32 numFreeVars()      const { return assign_.free()-1; }
-
-	//! Returns the number of constraints that are currently in the solver's learnt database.
-	uint32 numLearntConstraints() const { return (uint32)learnts_.size(); }
-	
+	uint32   numFreeVars()          const { return assign_.free()-1; }
 	//! Returns the value of v w.r.t the current assignment.
-	ValueRep value(Var v) const {
-		assert(validVar(v));
-		return assign_.value(v);
-	}
+	ValueRep value(Var v)           const { assert(validVar(v)); return assign_.value(v); }
 	//! Returns the previous value of v.
-	ValueRep savedValue(Var v) const {
-		assert(validVar(v));
-		return assign_.saved(v);
-	}
+	ValueRep savedValue(Var v)      const { assert(validVar(v)); return assign_.saved(v); }
+	//! Returns the preferred value of v.
+	ValueRep prefValue(Var v)       const { assert(validVar(v)); return assign_.pref(v); }
 	//! Returns true if p is true w.r.t the current assignment.
-	bool isTrue(Literal p) const {
-		assert(validVar(p.var()));
-		return assign_.value(p.var()) == trueValue(p);
-	}
+	bool     isTrue(Literal p)      const { assert(validVar(p.var())); return assign_.value(p.var()) == trueValue(p); }
 	//! Returns true if p is false w.r.t the current assignment.
-	bool isFalse(Literal p) const {
-		assert(validVar(p.var()));
-		return assign_.value(p.var()) == falseValue(p);
-	}
-	//! Returns the decision level on which v was assigned.
-	/*!
-	 * \note The returned value is only meaningful if value(v) != value_free.
-	 */
-	uint32 level(Var v) const {
-		assert(validVar(v));
-		return assign_.level(v);
-	}
-
-	//! Returns the reason for p being true.
-	/*!
-	 * \pre p is true w.r.t the current assignment
-	 */
-	const Antecedent& reason(Literal p) const {
-		assert(isTrue(p));
-		return assign_.reason(p.var());
-	}
-	//! Returns the reason for p being true as a set of literals.
-	void reason(Literal p, LitVec& out) {
-		assert(isTrue(p)); out.clear();
-		return assign_.reason(p.var()).reason(*this, p, out);
-	}
-	//! Returns the additional reason data associated with p.
-	uint32 reasonData(Literal p) const {
-		return assign_.data(p.var());
-	}
-	//! Updates the reason for p being tue.
-	/*!
-	 * \pre p is true and x is a valid reason for p
-	 */
-	bool setReason(Literal p, const Antecedent& x, uint32 data = UINT32_MAX) {
-		assert(isTrue(p));
-		assign_.setReason(p.var(), x);
-		if (data != UINT32_MAX) { assign_.setData(p.var(), data); }
-		return true;
-	}
+	bool     isFalse(Literal p)     const { assert(validVar(p.var())); return assign_.value(p.var()) == falseValue(p); }
 	//! Returns the literal of v being true in the current assignment.
 	/*!
 	 * \pre v is assigned a value in the current assignment
 	 */
-	Literal trueLit(Var v) const {
-		assert(value(v) != value_free);
-		return Literal(v, value(v) == value_false);
-	}
-
+	Literal  trueLit(Var v)         const { assert(value(v) != value_free); return Literal(v, valSign(value(v))); }
+	//! Returns the decision level on which v was assigned.
+	/*!
+	 * \note The returned value is only meaningful if value(v) != value_free.
+	 */
+	uint32   level(Var v)           const { assert(validVar(v)); return assign_.level(v); }
 	//! Returns true if v is currently marked as seen.
 	/*!
 	 * Note: variables assigned on level 0 are always marked.
 	 */
-	bool seen(Var v) const {
-		assert(validVar(v));
-		return assign_.seen(v, 3u);
-	}
+	bool     seen(Var v)            const { assert(validVar(v)); return assign_.seen(v, 3u); }
 	//! Returns true if the literal p is currently marked as seen.
-	bool seen(Literal p) const {
-		assert(validVar(p.var()));
-		return assign_.seen(p.var(), uint8(1+p.sign()));
-	}
-
+	bool     seen(Literal p)        const { assert(validVar(p.var())); return assign_.seen(p.var(), uint8(1+p.sign())); }
 	//! Returns the current decision level.
-	uint32 decisionLevel() const { return (uint32)levels_.size(); }
-
+	uint32   decisionLevel()        const { return (uint32)levels_.size(); }
+	bool     validLevel(uint32 dl)  const { return dl != 0 && dl <= decisionLevel(); }
 	//! Returns the starting position of decision level dl in the trail.
 	/*!
 	 * \pre validLevel(dl)
 	 */
-	uint32 levelStart(uint32 dl) const { assert(validLevel(dl)); return levels_[dl-1].trailPos; }
-
+	uint32   levelStart(uint32 dl)  const { assert(validLevel(dl)); return levels_[dl-1].trailPos; }
 	//! Returns the decision literal of the decision level dl.
 	/*!
 	 * \pre validLevel(dl)
 	 */
-	Literal decision(uint32 dl) const { assert(validLevel(dl)); return assign_.trail[ levels_[dl-1].trailPos ]; }
-
+	Literal  decision(uint32 dl)    const { assert(validLevel(dl)); return assign_.trail[ levels_[dl-1].trailPos ]; }
+	//! Returns true, if the current assignment is conflicting.
+	bool     hasConflict()          const { return !conflict_.empty(); }
+	//! Returns the number of (unprocessed) literals in the propagation queue.
+	uint32   queueSize()            const { return (uint32) assign_.qSize(); }
+	//! Number of problem constraints in this solver.
+	uint32   numConstraints()       const;
+	//! Returns the number of constraints that are currently in the solver's learnt database.
+	uint32   numLearntConstraints() const { return (uint32)learnts_.size(); }
+	//! Returns the reason for p being true.
+	/*!
+	 * \pre p is true w.r.t the current assignment
+	 */
+	const Antecedent& reason(Literal p)              const { assert(isTrue(p)); return assign_.reason(p.var()); }
+	//! Returns the additional reason data associated with p.
+	uint32            reasonData(Literal p)          const { return assign_.data(p.var()); }
 	//! Returns the current (partial) assignment as a set of true literals.
 	/*!
 	 * \note Although the special var 0 always has a value it is not considered to be
 	 * part of the assignment.
 	 */
-	const LitVec&     trail()      const { return assign_.trail; }
-	const Assignment& assignment() const { return assign_; }
-	
-	//! Returns true, if the current assignment is conflicting.
-	bool hasConflict() const { return !conflict_.empty(); }
+	const LitVec&     trail()                        const { return assign_.trail; }
+	const Assignment& assignment()                   const { return assign_; }
 	//! Returns the current conflict as a set of literals.
-	const LitVec& conflict() const { return conflict_; }
+	const LitVec&     conflict()                     const { return conflict_; }
 	//! Returns the most recently derived conflict clause.
-	const LitVec& conflictClause() const { return cc_; }
-	
-	//! Returns the number of (unprocessed) literals in the propagation queue.
-	uint32 queueSize()    const { return (uint32) assign_.qSize(); }
-	
+	const LitVec&     conflictClause()               const { return cc_; }
 	//! Returns the enumeration constraint set by the enumerator used.
-	Constraint* getEnumerationConstraint() const { return enum_; }
+	Constraint*       getEnumerationConstraint()     const { return enum_; }
+	//! Returns the idx'th learnt constraint.
+	/*!
+	 * \pre idx < numLearntConstraints()
+	 */
+	LearntConstraint& getLearnt(uint32 idx)          const {
+		assert(idx < numLearntConstraints());
+		return *static_cast<LearntConstraint*>(learnts_[ idx ]);
+	}
 	
 	SolveStats  stats;  /**< stores some statistics about the solving process */
 	//@}
@@ -695,16 +615,15 @@ public:
 	 */
 	//@{
 	//! Returns the number of constraints watching the literal p.
-	uint32  numWatches(Literal p) const;
+	uint32        numWatches(Literal p)              const;
 	//! Returns true if the constraint c watches the literal p.
-	bool    hasWatch(Literal p, Constraint* c) const;
-	bool    hasWatch(Literal p, ClauseHead* c) const;
+	bool          hasWatch(Literal p, Constraint* c) const;
+	bool          hasWatch(Literal p, ClauseHead* c) const;
 	//! Returns c's watch-structure associated with p.
 	/*!
 	 * \note returns 0, if hasWatch(p, c) == false
 	 */
 	GenericWatch* getWatch(Literal p, Constraint* c) const;
-
 	//! Adds c to the watch-list of p.
 	/*!
 	 * When p becomes true, c->propagate(p, data, *this) is called.
@@ -714,20 +633,17 @@ public:
 		assert(validWatch(p));
 		watches_[p.index()].push_right(GenericWatch(c, data));
 	}
-
 	//! Adds w to the clause watch-list of p.
 	void addWatch(Literal p, const ClauseWatch& w) {
 		assert(validWatch(p));
 		watches_[p.index()].push_left(w);
 	}
-	
 	//! Removes c from p's watch-list.
 	/*!
 	 * \post hasWatch(p, c) == false
 	 */
 	void removeWatch(const Literal& p, Constraint* c);
 	void removeWatch(const Literal& p, ClauseHead* c);
-
 	//! Adds c to the watch-list of decision-level dl.
 	/*!
 	 * Constraints in the watch-list of a decision level are
@@ -743,7 +659,6 @@ public:
 			levels_[dl-1].undo = allocUndo(c);
 		}
 	}
-	
 	//! Removes c from the watch-list of the decision level dl.
 	bool removeUndoWatch(uint32 dl, Constraint* c);
 	//@}
@@ -759,21 +674,77 @@ public:
 		assert(validVar(v));
 		if (assign_.saved(v) == value_free) { assign_.setSavedValue(v, val); }
 	}
-	//! Computes the literal block distance of p's antecedent given as the range [first, last).
+	//! Sets the preferred value of v to val. 
 	/*!
-	 * \note Returns strict lbd (i.e. lbd+1) if strategies().updateLbd > 1
+	 * The preferred value overwrites any sign heuristic.
 	 */
-	uint32 computeLbd(Literal p, const Literal* first, const Literal* last);
+	void initPrefValue(Var v, ValueRep val) {
+		assert(validVar(v));
+		if (assign_.pref(v) == value_free) { assign_.setPrefValue(v, val); }
+	}
+	//! Returns the default decision literal of v.
+	Literal  defaultLiteral(Var v);
+	//! Updates the reason for p being tue.
+	/*!
+	 * \pre p is true and x is a valid reason for p
+	 */
+	bool setReason(Literal p, const Antecedent& x, uint32 data = UINT32_MAX) {
+		assert(isTrue(p));
+		assign_.setReason(p.var(), x);
+		if (data != UINT32_MAX) { assign_.setData(p.var(), data); }
+		return true;
+	}
+	//! Returns the reason for p being true as a set of literals.
+	void reason(Literal p, LitVec& out) { assert(isTrue(p)); out.clear(); return assign_.reason(p.var()).reason(*this, p, out); }
+	//! Computes a new lbd for the antecedent of p given as the range [first, last).
+	/*!
+	 * \param p A literal implied by [first, last)
+	 * \param [first, last) The literals of a learnt nogood implying p.
+	 * \param cLbd The current lbd of the learnt nogood.
+	 * \return The new lbd of the learnt nogood.
+	 *
+	 * \note If SolverStrategies::bumpVarAct is active, p's activity
+	 *       is increased if the new lbd is smaller than the lbd of the
+	 *       conflict clause that is currently being derived.
+	 */
+	uint32 updateLearnt(Literal p, const Literal* first, const Literal* last, uint32 cLbd, bool forceUp = false) {
+		uint32 up = strategies().updateLbd;
+		if ((up || forceUp) && cLbd > 1) {
+			uint32 strict = (up == 2u);
+			uint32 p1     = (up == 3u);
+			uint32 nLbd   = (strict|p1) + countLevels(first, last, cLbd - strict);
+			if (nLbd < cLbd) { cLbd = nLbd - p1; }
+		}
+		if (strategies().bumpVarAct && isTrue(p)) { bumpAct_.push_back(WeightLiteral(p, cLbd)); }
+		return cLbd;
+	}
 	//! Visitor function for antecedents used during conflict clause minimization.
 	bool ccMinimize(Literal p, CCMinRecursive* rec) const {
 		return seen(p.var()) || 
 			(rec && hasLevel(level(p.var())) && rec->checkRecursive(p));
 	}
+	
+	//! Sets a soft memory limit on learnt nogoods.
+	/*!
+	 * The soft limit is checked in search() and when hit
+	 * search() will return value_free.
+	 */
+	void  setMemLimit(uint32 learntLimitInMB);
+	//! Returns true if number of learnts exceeds x.learnts or the soft memory limit is exceeded.
+	bool  learntLimit(const SearchLimits& x) const { return numLearntConstraints() > x.learnts || memLimit_ < 0; }
+	
 	//! Allocates a small block (32-bytes) from the solver's small block pool.
 	void* allocSmall()       { return smallAlloc_->allocate(); }
 	//! Frees a small block previously allocated from the solver's small block pool.
 	void  freeSmall(void* m) { smallAlloc_->free(m);    }
 	
+	void  addLearntBytes(uint32 bytes)  { memLimit_ -= bytes; }
+	void  freeLearntBytes(uint32 bytes) { memLimit_ += bytes; }
+	
+	//! simplifies cc and returns finalizeConflictClause(cc, info);
+	uint32  simplifyConflictClause(LitVec& cc, ClauseInfo& info, ClauseHead* rhs);
+	uint32  finalizeConflictClause(LitVec& cc, ClauseInfo& info);
+	uint32  countLevels(const Literal* first, const Literal* last, uint32 maxLevels);
 	bool hasLevel(uint32 dl)    const { assert(validLevel(dl)); return levels_[dl-1].marked != 0; }
 	bool frozenLevel(uint32 dl) const { assert(validLevel(dl)); return levels_[dl-1].freeze != 0; }
 	void markLevel(uint32 dl)    { assert(validLevel(dl)); levels_[dl-1].marked = 1; }
@@ -796,20 +767,20 @@ private:
 		uint32        freeze   :  1;
 		ConstraintDB* undo;
 	};
-	typedef PodVector<DLevel>::type         DecisionLevels;  
-	typedef PodVector<Antecedent>::type     ReasonVec;
-	typedef PodVector<WatchList>::type      Watches;
+	typedef PodVector<DLevel>::type     DecisionLevels;  
+	typedef PodVector<Antecedent>::type ReasonVec;
+	typedef PodVector<WatchList>::type  Watches;
 	struct PPList {
 		PPList();
 		~PPList();
 		void add(PostPropagator* p);
 		void remove(PostPropagator* p);
-		bool propagate(Solver& s, PostPropagator* p);
+		bool propagate(Solver& s, PostPropagator* p) const;
 		void simplify(Solver& s, bool shuf);
-		void reset();
-		bool isModel(Solver& s);
-		bool init(Solver& s, PostPropagator* list);
-		bool init(Solver& s) { return init(s, head); }
+		void reset() const;
+		bool isModel(Solver& s) const;
+		bool init(Solver& s, PostPropagator* list) const;
+		bool init(Solver& s) const { return init(s, head); }
 		PostPropagator* enableFunky();
 		PostPropagator* disableFunky();
 		PostPropagator* nil() const { return (PostPropagator*)1; }
@@ -817,7 +788,6 @@ private:
 		PostPropagator* look;
 	};
 	struct CmpScore {
-		typedef LearntConstraint::Activity Activity;
 		typedef std::pair<uint32, Activity> ViewPair;
 		CmpScore(const ConstraintDB& learnts, ReduceStrategy::Score sc, uint32 g) : db(learnts), rs(sc), glue(g) {}
 		uint32 score(const Activity& act)                             const { return ReduceStrategy::asScore(rs, act); }
@@ -836,50 +806,53 @@ private:
 	};
 	inline  bool hasStopConflict() const;
 	bool    validWatch(Literal p) const { return p.index() < (uint32)watches_.size(); }
-	uint32  mark(uint32 s, uint32 e);
 	void    freeMem();
 	bool    simplifySAT();
 	void    simplifyShort(Literal p);
 	bool    unitPropagate();
+	bool    propagateUnits();
 	void    undoLevel(bool sp);
 	uint32  analyzeConflict();
 	void    otfs(Antecedent& lhs, const Antecedent& rhs, Literal p, bool final);
 	ClauseHead* otfsRemove(ClauseHead* c, const LitVec* newC);
-	uint32  finalizeConflictClause(ClauseHead* rhs);
-	uint32  ccMinimize(LitVec& cc, LitVec& removed, uint32 anteMask, CCMinRecursive* ccMin);
-	bool    ccRemovable(Literal p, uint32 anteMask, CCMinRecursive* ccMin);
-	bool    ccReverseArc(Literal p, uint32 maxLevel, uint32 maxNew, Antecedent& out);
+	uint32  ccMinimize(LitVec& cc, LitVec& removed, uint32 antes, CCMinRecursive* ccMin);
+	bool    ccRemovable(Literal p, uint32 antes, CCMinRecursive* ccMin);
+	Antecedent ccHasReverseArc(Literal p, uint32 maxLevel, uint32 maxNew);
+	void    ccResolve(LitVec& cc, uint32 pos, const LitVec& reason);
 	void    undoFree(ConstraintDB* x);
 	void    setConflict(Literal p, const Antecedent& a, uint32 data);
-	DBInfo  reduceLinear(uint32 minR, uint32 maxR, const CmpScore& cmp);
-	DBInfo  reduceSorted(uint32 minR, uint32 maxR, const CmpScore& cmp);
-	DBInfo  reduceSortedInPlace(uint32 minR, uint32 maxR, const CmpScore& cmp);
+	uint64  updateBranch(uint32 cfl);
+	DBInfo  reduceLinear(uint32 maxR, const CmpScore& cmp);
+	DBInfo  reduceSort(uint32 maxR, const CmpScore& cmp);
+	DBInfo  reduceSortInPlace(uint32 maxR, const CmpScore& cmp, bool onlyPartialSort);
 	ConstraintDB* allocUndo(Constraint* c);
 	SolverStrategies  strategy_;    // Strategies used by this solver-object
+	Heuristic         heuristic_;   // active decision heuristic
 	Assignment        assign_;      // three-valued assignment
 	ConstraintDB      constraints_; // problem constraints
 	ConstraintDB      learnts_;     // learnt constraints
 	ImpliedList       impliedLits_; // Lits that were asserted on current dl but are logically implied earlier
 	LitVec            conflict_;    // stores conflict-literals for later analysis
 	LitVec            cc_;          // temporary: stores conflict clause within analyzeConflict
-	LitVec            unconstr_;    // eliminated vars that are unconstraint w.r.t the current model
+	LitVec            temp_;        // temporary: eliminated vars that are unconstraint w.r.t the current model
+	WeightLitVec      bumpAct_;     // temporary: lits from current dl whose activity might get an extra bump
 	DecisionLevels    levels_;      // Stores information (e.g. position in trail) on each decision level
 	VarVec            lbdStamp_;    // temporary vector for computing LBD
+	VarVec            cflStamp_;    // temporary vector for computing number of conflicts in branch
 	Watches           watches_;     // for each literal p: list of constraints watching p
 	PPList            post_;        // (possibly empty) list of post propagators
 	ClauseInfo        ccInfo_;      // temporary: stores information about conflict clause cc_
 	CCMinRecursive*   ccMin_;       // additional data for supporting recursive strengthen
 	SmallClauseAlloc* smallAlloc_;  // allocator object for small clauses
 	SharedContext*    shared_;      // initialized by master thread - otherwise read-only!
-	VarVec*           levConflicts_;// For a DL d, levConflicts_[d-1] stores num conflicts when d was started
 	ConstraintDB*     undoHead_;    // Free list of undo DBs
 	Constraint*       enum_;        // enumeration constraint - set by enumerator
+	int64             memLimit_;    // Soft memory limit on learnt constraints
 	uint32            id_;          // solver id - only needed with multi-threading
 	uint32            units_;       // number of top-level assignments: always marked as seen
 	LitVec::size_type lastSimplify_;// number of top-level assignments on last call to simplify
 	uint32            rootLevel_;   // dl on which search started.
 	uint32            btLevel_;     // When enumerating models: DL of the last unflipped decision from the current model. Can't backjump below this level.
-	uint32            tagged_;      // number of conditional clauses
 	uint32            lbdTime_;     // temporary counter for computing lbd
 	bool              shuffle_;     // shuffle program on next simplify?
 };
@@ -887,6 +860,12 @@ private:
 inline bool isRevLit(const Solver& s, Literal p, uint32 maxL) {
 	return s.isFalse(p) && (s.seen(p) || s.level(p.var()) < maxL);
 }
+inline Literal Solver::defaultLiteral(Var v) {
+	uint32 xs = strategies().signDef;
+	if (xs == SolverStrategies::sign_type) { return sharedContext()->preferredLiteralByType(v); }
+	if (xs == SolverStrategies::sign_rnd)  { return Literal(v, strategies().rng.drand() < 0.5); }
+	return Literal(v, xs == SolverStrategies::sign_yes);
+}
 
 //@}
 
@@ -905,7 +884,6 @@ class DecisionHeuristic {
 public:
 	DecisionHeuristic() {}
 	virtual ~DecisionHeuristic(); 
-	
 	/*!
 	 * Called once after all problem variables are known to the solver.
 	 * The default-implementation is a noop.
@@ -920,14 +898,6 @@ public:
 	 * \param s The solver in which this heuristic is used.
 	 */
 	virtual void endInit(Solver& /* s */) { }  
-
-	/*!
-	 * If the heuristic is used in an incremental setting, enable/disable
-	 * reinitialization of existing variables.
-	 * The default-implementation is a noop. Hence, heuristics will typically
-	 * simply (re-)initialize all variables.
-	 */
-	virtual void reinit(bool /* b */) {}
 	
 	/*!
 	 * Called for each var that changes its state from eliminated back to normal.
@@ -1019,13 +989,12 @@ public:
 	virtual Literal selectRange(Solver& /* s */, const Literal* first, const Literal* /* last */) {
 		return *first;
 	}
-protected:
-	Literal savedLiteral(const Solver& s, Var var) const {
-		Literal r(0,false); ValueRep v;
-		if ((v = s.savedValue(var)) != value_free) {
-			r = Literal(var, v == value_false);
-		}
-		return r;
+	static Literal selectLiteral(Solver& s, Var v, int sc) {
+		ValueRep xv;
+		if ((xv = s.prefValue(v)) != value_free) { return Literal(v, valSign(xv)); }
+		if ((xv = s.savedValue(v))!= value_free) { return Literal(v, valSign(xv)); }
+		if (sc != 0)                             { return Literal(v, sc < 0); }
+		return s.defaultLiteral(v);
 	}
 private:
 	DecisionHeuristic(const DecisionHeuristic&);
