@@ -20,7 +20,7 @@
 #include <clasp/cb_enumerator.h>
 #include <clasp/solver.h>
 #include <clasp/clause.h>
-#include <clasp/util/thread.h>
+#include <clasp/util/mutex.h>
 #include <stdio.h> // sprintf
 #ifdef _MSC_VER
 #pragma warning (disable : 4996) // sprintf may be unfase
@@ -42,12 +42,12 @@ public:
 			ret = ClauseCreator::integrate(s, createShared(c), flags);
 		}
 		else {
-			ret = ClauseCreator::create(s, c, flags, ClauseInfo().setType(Constraint_t::learnt_other));
+			ret = ClauseCreator::create(s, c, flags, ClauseInfo(Constraint_t::learnt_other));
 		}
 		return ret.local;
 	}
 	SharedLiterals* getShared() { 
-		tbb::spin_mutex::scoped_lock lock(mutex_);
+		std::lock_guard<tbb::spin_mutex> lock(mutex_);
 		return current_ ? current_->share() : 0;
 	}
 	bool shared() const { return shared_; }
@@ -55,7 +55,7 @@ private:
 	SharedLiterals* createShared(const LitVec& c) {
 		SharedLiterals* newShared = SharedLiterals::newShareable(c, Constraint_t::learnt_other, 2);
 		SharedLiterals* oldShared = current_;
-		{ tbb::spin_mutex::scoped_lock lock(mutex_);
+		{ std::lock_guard<tbb::spin_mutex> lock(mutex_);
 			current_ = newShared;
 		}
 		if (oldShared) oldShared->release();
@@ -63,7 +63,7 @@ private:
 	}
 	tbb::spin_mutex mutex_;
 	SharedLiterals* current_;
-	bool					  shared_;
+	bool            shared_;
 };
 /////////////////////////////////////////////////////////////////////////////////////////
 // CBConsequences::LocalConstraint
@@ -241,16 +241,15 @@ void CBConsequences::LocalConstraint::add(Solver& s, Constraint* c) {
 }
 bool CBConsequences::LocalConstraint::integrate(Solver& s, SharedLiterals* clause) {
 	if (!clause) return true;
-	ClauseCreator::Result ret;
+	ClauseCreator::Result ret(0, ClauseCreator::status_conflicting);
 	if (clause->size() > 0) {
 		ret = ClauseCreator::integrate(s, clause, ClauseCreator::clause_explicit|ClauseCreator::clause_no_add);
 		if (ret.local) { add(s, ret.local); }
 	}
 	else {
 		s.setStopConflict();
-		ret.ok = false;
 	}
-	return ret.ok;
+	return ret.ok();
 }
 
 }
