@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2007, Benjamin Kaufmann
+// Copyright (c) 2009, 2012 Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -207,11 +207,11 @@ bool Lookahead::propagate(Solver& s) {
 }
 
 bool Lookahead::propagateFixpoint(Solver& s) {
-	if (empty() || top_ == s.numAssignedVars()) {
+	if ((empty() || top_ == s.numAssignedVars()) && !score.deps.empty()) {
 		// nothing to lookahead
 		return true;
 	}
-	bool ok   = true;
+	bool ok = true;
 	uint32 dl;
 	for (dl = s.decisionLevel(); !propagate(s); dl = s.decisionLevel()) {
 		// some literal failed
@@ -227,8 +227,7 @@ bool Lookahead::propagateFixpoint(Solver& s) {
 		// remember top-level size - no need to redo lookahead 
 		// on level 0 unless we learn a new implication
 		assert(s.queueSize() == 0);
-		top_    = s.numAssignedVars();
-		tlc_    = Literal(score.best, score.score[score.best].prefSign());
+		top_ = s.numAssignedVars();
 		LitVec().swap(imps_);
 	}	
 	return ok;
@@ -278,36 +277,19 @@ void Lookahead::undoLevel(Solver& s) {
 		saved_.pop_back();
 		splice(n);
 		assert(node(last_)->next == head_id);
+		score.clearDeps();
 	}
 }
 
 Literal Lookahead::heuristic(Solver& s) {
-	if (s.numFreeVars() == 0) { 
-		return negLit(0);  
+	if (s.value(score.best) != value_free) {
+		// no candidate available
+		return posLit(0);
 	}
-	ScoreLook& sc  = score;
-	Literal choice = Literal(sc.best, sc.score[sc.best].prefSign());
-	if (sc.deps.empty()) {
-		// No candidates. This can happen if the problem 
-		// contains choice rules and lookahead is not atom-based.
-		// Add remaining free vars to lookahead so that we can
-		// make an informed decision.
-		for (Var v = 1, end = s.numVars()+1; v != end; ++v) {
-			if (s.value(v) == value_free) {
-				append(s.sharedContext()->preferredLiteralByType(v), true);
-			}
-		}
-		sc.types  = Var_t::atom_body_var;
-		uint32 dl = s.decisionLevel();
-		if (!propagate(s)) {
-			Literal choice = s.decision(s.decisionLevel());
-			s.undoUntil(dl);
-			return choice;
-		}
-		choice = Literal(sc.best, sc.score[sc.best].prefSign());
-	}
-	assert(s.value(choice.var()) == value_free);
-	if (sc.mode == ScoreLook::score_max_min) {
+	ScoreLook& sc = score;
+	Literal choice= Literal(sc.best, sc.score[sc.best].prefSign());
+	if (!sc.deps.empty() && sc.mode == ScoreLook::score_max_min) {
+		// compute heuristic values for candidates skipped during last lookahead
 		uint32 min, max;
 		sc.score[sc.best].score(max, min);
 		sc.addDeps = false;
@@ -316,7 +298,7 @@ Literal Lookahead::heuristic(Solver& s) {
 		do {
 			Var v        = sc.deps[i];
 			VarScore& vs = sc.score[v];
-			if (!vs.testedBoth() && s.value(v) == value_free) {
+			if (s.value(v) == value_free) {
 				uint32 vMin, vMax;
 				vs.score(vMax, vMin);
 				if (vMin == 0 || vMin > min || (vMin == min && vMax > max)) {
@@ -336,7 +318,6 @@ Literal Lookahead::heuristic(Solver& s) {
 					choice = Literal(v, vs.prefSign());
 				}
 			}
-			vs.clear();
 		} while (++i != sc.deps.size() && ok); 
 		if (!ok) {
 			// One of the candidates failed. Since none of them failed
@@ -345,42 +326,57 @@ Literal Lookahead::heuristic(Solver& s) {
 			// parallel solving is active and a stop conflict was set.
 			// Since we can't resolve the problem here, we simply return the
 			// literal that caused the conflict
-			sc.clearDeps();
-			assert(s.hasConflict() && s.decisionLevel() > 0);
-			choice = negLit(0);
-		}
-		sc.deps.clear();	
+			assert(s.hasConflict());
+			return negLit(0);
+		}		
 	}
-	if (s.decisionLevel() == 0) { tlc_ = choice; }
 	return choice;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////
 // Lookahead heuristic
 /////////////////////////////////////////////////////////////////////////////////////////
-UnitHeuristic::UnitHeuristic(Lookahead::Type t, bool nant)
+UnitHeuristic::UnitHeuristic(Lookahead::Type t)
 	: look_(0)
-	, type_(t) 
-	, init_(false)
-	, nant_(nant && t == Lookahead::atom_lookahead) {
+	, type_(t) {
 }
 
 UnitHeuristic::~UnitHeuristic() { }
 
 void UnitHeuristic::endInit(Solver& s) { 
 	assert(s.decisionLevel() == 0);
-	if (init_ && look_) {
+	if (s.strategies().heuReinit && look_) {
 		look_->destroy(&s, true);
 		look_ = 0;
 	}
 	if (look_ == 0) { 
 		look_ = new Lookahead(type_);
-		look_->score.nant = nant_;
+		look_->score.nant = s.strategies().unitNant && type_ == Lookahead::atom_lookahead;
 		look_->init(s);
 		s.addPost(look_);
 	}
 }
 
+Literal UnitHeuristic::doSelect(Solver& s) {
+	Literal x = look_->heuristic(s);
+	if (x != posLit(0) || s.numFreeVars() == 0) { return x; }
+	// No candidates. This can happen if the problem 
+	// contains choice rules and lookahead is not atom-based.
+	// Add remaining free vars to lookahead so that we can
+	// make an informed decision.
+	VarType types = look_->score.types;
+	for (Var v = 1, end = s.numVars()+1; v != end; ++v) {
+		if ((s.value(v) == value_free || s.level(v) > 0) && (s.sharedContext()->type(v) & types) == 0) {
+			look_->append(s.sharedContext()->preferredLiteralByType(v), true);
+		}
+	}
+	look_->score.clearDeps();
+	look_->score.types = Var_t::atom_body_var;
+	return look_->propagateFixpoint(s) 
+		? look_->heuristic(s)
+		: negLit(0);
+}
+
 void UnitHeuristic::resurrect(const Solver& s, Var v) {
 	if ( look_ && (s.sharedContext()->type(v) & look_->score.types) != 0 ) {
 		look_->score.score.resize(s.numVars()+1);
@@ -398,8 +394,8 @@ RestrictedUnit::RestrictedUnit(uint32 k, Lookahead* look, DecisionHeuristic* def
 
 void RestrictedUnit::decorate(Solver& s, uint32 k, Lookahead* look) {
 	s.removePost(look);
-	RestrictedUnit* unit = new RestrictedUnit(k, look, s.strategies().heuristic.release());
-	s.strategies().heuristic.reset(unit);
+	RestrictedUnit* unit = new RestrictedUnit(k, look, s.heuristic().release());
+	s.heuristic().reset(unit);
 }
 
 RestrictedUnit::~RestrictedUnit() {
@@ -409,13 +405,20 @@ RestrictedUnit::~RestrictedUnit() {
 
 void RestrictedUnit::endInit(Solver& s) {
 	destroy(s);
-	s.strategies().heuristic->endInit(s);
+	s.heuristic()->endInit(s);
 }
 
 Literal RestrictedUnit::doSelect(Solver& s) {
 	s.addPost(look_);
 	Literal choice = look_->propagateFixpoint(s) ? look_->heuristic(s) : negLit(0);
 	s.removePost(look_);
+	// if all fld-candidates are currently assigned, use
+	// decorated heuristic to select next choice -
+	// if all vars are assigned after fld, we return posLit(0)
+	// as the "noop" choice.
+	if (choice == posLit(0) && s.numFreeVars() > 0) {
+		choice = default_->doSelect(s);
+	}
 	if (!isSentinel(choice) && --numChoices_ == 0) {
 		destroy(s);
 	}
@@ -423,10 +426,10 @@ Literal RestrictedUnit::doSelect(Solver& s) {
 }
 
 void RestrictedUnit::destroy(Solver& s) {
-	if (s.strategies().heuristic.get() == this) {
-		s.strategies().heuristic.release();
+	if (s.heuristic().get() == this) {
+		s.heuristic().release();
 	}
-	s.strategies().heuristic.reset(default_);
+	s.heuristic().reset(default_);
 	default_ = 0;
 	if (look_) { look_->destroy(&s, true); look_ = 0;  } 
 	delete this;
