@@ -388,15 +388,17 @@ bool MinimizeConstraint::implied(wsum_t* lhs, weight_t wOrIdx) {
 /////////////////////////////////////////////////////////////////////////////////////////
 SharedMinimizeData::SharedMinimizeData(const SumVec& lhsSum, const SumVec& lhsAdjust, const SumVec& rhsSum, MinimizeMode m) : mode_(m) {
 	sum_         = lhsSum;
-	adjust_      = lhsAdjust.empty() ? 0 : new wsum_t[lhsAdjust.size()];
-	std::copy(lhsAdjust.begin(), lhsAdjust.end(), adjust_);
 	opts_[0].seq = 0;
 	opts_[0].opt = rhsSum;
 	opts_[0].step= Optimum::Step(numRules()-1, 0);
 	opt_         = &opts_[0];
 	next_        = &opts_[1];
+	adjust_      = 0;
 	low_         = -static_cast<int64>(UINT64_MAX/2);
 	hierarch_    = 0;
+	if (!lhsAdjust.empty()) {
+		std::copy(lhsAdjust.begin(), lhsAdjust.end(), (adjust_ = new wsum_t[lhsAdjust.size()]));
+	}
 }
 SharedMinimizeData::~SharedMinimizeData() {
 	delete [] adjust_;
