@@ -374,7 +374,7 @@ bool ParallelSolve::doSolve(Solver& s, const SolveParams& p) {
 // main solve loop executed by all threads
 void ParallelSolve::solveParallel(uint32 id) {
 	Solver& s           = thread_[id]->solver();
-	const SolveParams p = thread_[id]->params();
+	const SolveParams& p= thread_[id]->params();
 	InitParams init     = p.init;
 	SolveLimits lim     = getSolveLimits();
 	PathPtr a(0);
@@ -666,7 +666,9 @@ ParallelHandler::ParallelHandler(ParallelSolve& ctrl, Solver& s, const SolvePara
 	this->next = this;
 }
 
-ParallelHandler::~ParallelHandler() { clearDB(0); }
+ParallelHandler::~ParallelHandler() { 
+	clearDB(0); 
+}
 
 // adds this as post propagator to its solver and attaches the solver to ctx.
 bool ParallelHandler::attach(SharedContext& ctx) {
@@ -695,7 +697,11 @@ void ParallelHandler::detach(SharedContext& ctx, bool fastExit) {
 		}
 		aggStats.accu(solver_->stats);
 		ctx.detach(*solver_);
-		if (error()) { solver_->reset(); solver_->setId(Solver::invalidId); }
+		if (error()) { 
+			clearDB(0);
+			solver_->reset();
+			solver_->setId(Solver::invalidId); 
+		}
 		solver_->stats.swapStats(aggStats);
 		solver_->stats.parallel->cpuTime = ThreadTime::getTime();
 	}
