@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006, 2007, 2012 Benjamin Kaufmann
 //
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
 //
@@ -1505,7 +1505,7 @@ void ProgramBuilder::updateFrozenAtoms() {
 		incData_->unfreeze_.erase(j, incData_->unfreeze_.end());
 		// add protection for atoms still frozen
 		j = incData_->freeze_.begin();
-		Body emptyBody(0,0);
+		Body emptyBody(static_cast<PrgBodyNode*>(0),0);
 		for (VarVec::iterator it = j, end = incData_->freeze_.end(); it != end; ++it) {
 			PrgAtomNode* a = atoms_[*it];
 			if (a->frozen()) {
