@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -75,40 +75,19 @@ void StreamSource::underflow() {
 	buffer_[in_.gcount()] = 0;
 }
 
-char StreamSource::operator*() {
-	if (buffer_[pos_] == 0) {
-		underflow();
-	}
-	return buffer_[pos_];
-}
-
-StreamSource& StreamSource::operator++() {
-	if (buffer_[pos_++] == '\n') {++line_;}
-	**this;
-	return *this;
-}
-	
-bool StreamSource::parseInt64( int64& val) {
-	val = 0;
-	bool  pos = true;
-	skipWhite();
-	if (**this == '-') {
-		pos = false;
-		++*this;
-	}
-	if (**this == '+') {
-		++*this;
-	}
-	bool ok = **this >= '0' && **this <= '9';
-	int d;
-	while (**this >= '0' && **this <= '9') {
-		d    = **this - '0';
+bool StreamSource::parseInt64(int64& val) {
+	skipSpace();
+	bool pos = match('+') || !match('-');
+	int  d   = **this - '0';
+	if (d < 0 || d > 9) { return false; }
+	val      = 0;
+	do {
 		val *= 10;
 		val += d;
-		++*this;
-	}
+		d    = *(++*this) - '0';
+	} while (d >= 0 && d <= 9);
 	val  = pos ? val : -val;
-	return ok;
+	return true;
 }
 bool StreamSource::parseInt(int& val) { 
 	int64 x;
@@ -125,42 +104,34 @@ bool StreamSource::parseInt(int& val, int min, int max) {
 		&&   (val=(int)x, true);
 }
 
-void StreamSource::skipWhite() {
-	while ( **this == ' ' || **this == '\t' ) {
-		++*this;
+bool StreamSource::matchEol() {
+	if (match('\n')) {
+		++line_;
+		return true;
 	}
-}
-
-bool StreamSource::readLine( char* buf, unsigned maxSize ) {
-	if (maxSize == 0) return false;
-	for (;;) {
-		char ch = **this;
-		++*this;
-		if ( ch == '\0' || ch == '\n' || --maxSize == 0) break;
-		*buf++ = ch;
+	if (match('\r')) {
+		match('\n');
+		++line_;
+		return true;
 	}
-	*buf = 0;
-	return maxSize != 0;
+	return false;
 }
 
 bool readLine(StreamSource& in, PodVector<char>::type& buf ) {
-	char   buffer[1024];
+	char buffer[1024];
+	bool eol;
 	uint32 i;
 	buf.clear();
-	for (i = 0;;) {
+	for (i = 0; *in && (eol = in.matchEol()) == false; ++in) {
 		buffer[i] = *in;
-		++in;
-		if (buffer[i] == '\n' || buffer[i] == '\0') {
-			buf.insert(buf.end(), buffer, buffer+i+1);
-			buf.back() = '\0';
-			break;
-		}
 		if (++i == 1024) {
 			buf.insert(buf.end(), buffer, buffer+i);
 			i = 0;
 		}
 	}
-	return buffer[i] == '\n';
+	buf.insert(buf.end(), buffer, buffer+i);
+	buf.push_back('\0');
+	return eol;
 }
 	
 namespace {
@@ -181,6 +152,9 @@ public:
 	bool  endParse();
 	bool  readRule(int);
 	bool  readBody(uint32 lits, uint32 neg, bool weights);
+	bool  check(bool cond, const char* m) const {
+		return cond || (throw ReadError(source_->line(), m), false);
+	}
 	PrgRule         rule_;
 	StreamSource*   source_;
 	ProgramBuilder* api_;
@@ -225,55 +199,37 @@ Var LparseReader::parseAtom() {
 
 bool LparseReader::readRules() {
 	int rt = -1;
-	while ( skipAllWhite(*source_) && source_->parseInt(rt) && rt != 0 && readRule(rt) ) ;
-	if (rt != 0) {
-		throw ReadError(source_->line(), "Rule type expected!");
-	}
-	if (!match(*source_, '\n', true)) {
-		throw ReadError(source_->line(), "Symbol table expected!");
-	}
-	return skipAllWhite(*source_);
+	while ( source_->skipWhite() && source_->parseInt(rt) && rt != 0 && readRule(rt) ) ;
+	return check(rt == 0, "Rule type expected!")
+		&&   check(matchEol(*source_, true), "Symbol table expected!")
+		&&   source_->skipWhite();
 }
 
 bool LparseReader::readRule(int rt) {
 	int bound = -1;
-	if (rt <= 0 || rt > 6 || rt == 4) {
-		throw ReadError(source_->line(), "Unsupported rule type!");
-	}
+	check(rt > 0 && rt <= 6 && rt != 4, "Unsupported rule type!");
 	RuleType type(static_cast<RuleType>(rt));
 	rule_.setType(type);
 	if ( type == BASICRULE || rt == CONSTRAINTRULE || rt == WEIGHTRULE) {
 		rule_.addHead(parseAtom());
-		if (rt == WEIGHTRULE && !source_->parseInt(bound, 0, INT_MAX)) {
-			throw ReadError(source_->line(), "Weightrule: Positive weight expected!");
-		}
+		check(rt != WEIGHTRULE || source_->parseInt(bound, 0, INT_MAX), "Weightrule: Positive weight expected!");
 	}
 	else if (rt == CHOICERULE) {
-		int heads;
-		if (!source_->parseInt(heads, 1, INT_MAX)) {
-			throw ReadError(source_->line(), "Choicerule: To few heads");
-		}
+		int heads = 0;
+		check(source_->parseInt(heads, 1, INT_MAX), "Choicerule: To few heads");
 		for (int i = 0; i < heads; ++i) {
 			rule_.addHead(parseAtom());
 		}
 	}
 	else {
 		assert(rt == 6);
-		int x;
-		if (!source_->parseInt(x,0,0)) {
-			throw ReadError(source_->line(), "Minimize rule: 0 expected!");
-		}
-	}
-	int lits, neg;
-	if (!source_->parseInt(lits, 0, INT_MAX)) {
-		throw ReadError(source_->line(), "Number of body literals expected!");
-	}
-	if (!source_->parseInt(neg, 0, lits)) {
-		throw ReadError(source_->line(), "Illegal negative body size!");
-	}
-	if (rt == CONSTRAINTRULE && !source_->parseInt(bound, 0, INT_MAX)) {
-		throw ReadError(source_->line(), "Constraint rule: Positive bound expected!");
+		int x = 0;
+		check(source_->parseInt(x,0,0), "Minimize rule: 0 expected!");
 	}
+	int lits = 0, neg = 0;
+	check(source_->parseInt(lits, 0, INT_MAX), "Number of body literals expected!");
+	check(source_->parseInt(neg, 0, lits), "Illegal negative body size!");
+	check(rt != CONSTRAINTRULE || source_->parseInt(bound, 0, INT_MAX), "Constraint rule: Positive bound expected!");
 	if (bound >= 0) {
 		rule_.setBound(static_cast<uint32>(bound));
 	}
@@ -286,76 +242,48 @@ bool LparseReader::readBody(uint32 lits, uint32 neg, bool readWeights) {
 	}
 	if (readWeights) {
 		for (uint32 i = 0; i < lits; ++i) {
-			int w;
-			if (!source_->parseInt(w, 0, INT_MAX)) {
-				throw ReadError(source_->line(), "Weight Rule: bad or missing weight!");
-			}
+			int w = 0;
+			check(source_->parseInt(w, 0, INT_MAX), "Weight Rule: bad or missing weight!");
 			rule_.body[i].second = w;
 		}
 	} 
 	api_->addRule(rule_);
 	rule_.clear();
-	return match(*source_, '\n', true) ? true : throw ReadError(source_->line(), "Illformed rule body!");
+	return check(matchEol(*source_, true), "Illformed rule body!");
 }
 
 bool LparseReader::readSymbolTable() {
 	int a = -1;
 	PodVector<char>::type buf;
 	buf.reserve(1024);
-	while (source_->parseInt(a) && a != 0) {
-		if (a < 1) {
-			throw ReadError(source_->line(), "Symbol Table: Atom id out of bounds!");
-		}
-		source_->skipWhite();
-		if (!readLine(*source_, buf)) {
-			throw ReadError(source_->line(), "Symbol Table: Atom name too long or end of file!");
-		}
+	while (source_->skipWhite() && source_->parseInt(a) && a != 0) {
+		check(a >= 1, "Symbol Table: Atom id out of bounds!");
+		check(source_->skipSpace() && readLine(*source_, buf), "Symbol Table: Atom name too long or end of file!");
 		api_->setAtomName(a, &buf[0]);
-		skipAllWhite(*source_);
 	}
-	if (a != 0) {
-		throw ReadError(source_->line(), "Symbol Table: Atom id expected!");
-	}
-	if (!match(*source_, '\n', true)) {
-		throw ReadError(source_->line(), "Compute Statement expected!");
-	}
-	return skipAllWhite(*source_);
+	check(a == 0, "Symbol Table: Atom id expected!");
+	return check(matchEol(*source_, true), "Compute Statement expected!");
 }
 
 bool LparseReader::readComputeStatement() {
-	char pos[2] = { '+', '-' };
+	const char* B[2] = { "B+", "B-" };
 	for (int i = 0; i != 2; ++i) {
-		char sec[3];
-		skipAllWhite(*source_);
-		if (!source_->readLine(sec, 3) || sec[0] != 'B' || sec[1] != pos[i]) {
-			throw ReadError(source_->line(), (i == 0 ? "B+ expected!" : "B- expected!"));
-		}
-		skipAllWhite(*source_);
+		source_->skipWhite();
+		check(match(*source_, B[i], false) && matchEol(*source_, true), (i == 0 ? "B+ expected!" : "B- expected!"));
 		int id = -1;
-		while (source_->parseInt(id) && id != 0) {
-			if (id < 1) throw ReadError(source_->line(), "Compute Statement: Atom out of bounds");  
-			api_->setCompute(static_cast<Var>(id), pos[i] == '+');
-			if (!match(*source_, '\n', true)) {
-				throw ReadError(source_->line(), "Newline expected!");
-			}
-			skipAllWhite(*source_);
-		}
-		if (id != 0) {
-			throw ReadError(source_->line(), "Compute Statement: Atom id or 0 expected!");
-		}
-		if (!match(*source_, '\n', true)) {
-			throw ReadError(source_->line(), (i == 0 ? "B- expected!" : "Number of models expected!"));
+		while (source_->skipWhite() && source_->parseInt(id) && id != 0) {
+			check(id >= 1, "Compute Statement: Atom out of bounds");
+			api_->setCompute(static_cast<Var>(id), B[i][1] == '+');
 		}
+		check(id == 0, "Compute Statement: Atom id or 0 expected!");
 	}
-	return skipAllWhite(*source_);
+	return true;
 }
 
 bool LparseReader::readModels() {
-	int m;
-	if (!source_->parseInt(m, 0, INT_MAX)) {
-		throw ReadError(source_->line(), "Number of models expected!");
-	}
-	return skipAllWhite(*source_);
+	int m = 1;
+	check(source_->skipWhite() && source_->parseInt(m, 0, INT_MAX), "Number of models expected!");
+	return true;
 }
 
 bool LparseReader::endParse() {
@@ -403,7 +331,7 @@ private:
 	char* name_;
 };
 
-bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, ObjectiveFunction& opt, bool maxSat) {
+bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, PureLitMode pure, ObjectiveFunction& opt, bool maxSat) {
 	LitVec currentClause;
 	ClauseCreator nc(ctx.master());
 	SatPreprocessor* p = 0;
@@ -418,10 +346,11 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 	// c2: set if v occurs positively in the current clause
 	PodVector<uint8>::type  flags;
 	TempFile temp;
-	bool  wcnf = false, partial = false;
-	wsum_t top = 1;
+	bool  wcnf  = false, partial = false;
+	wsum_t top  = 1;
+	uint8  keep = 12u;
 	for (uint32 state = 0;ret;)  {
-		skipAllWhite(in);
+		in.skipWhite();
 		if      (*in == 'c')       { skipLine(in); }
 		else if (*in == 0 && state){ break; }
 		else if (state == 1)       { // read clause
@@ -434,7 +363,7 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 			for (;;){
 				if (!in.parseInt(lit))  { ERROR("Bad parameter in clause!"); }
 				if (abs(lit) > numVars) { ERROR("Unrecognized format - variables must be numbered from 1 up to $VARS!"); }
-				skipAllWhite(in);
+				in.skipWhite();
 				rLit = lit >= 0 ? posLit(lit) : negLit(-lit);
 				if (lit == 0) {
 					nc.start();
@@ -443,13 +372,15 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 						if (w != cw) { ERROR("Clause weight too large!"); }
 						// soft clause
 						if (currentClause.size() > 1) {
-							flags.push_back(0);
+							flags.push_back(keep); // not pure!
 							currentClause.push_back(posLit(++lastVar));
 							opt.lits.push_back(WeightLiteral(currentClause.back(), w));
 						}
 						else {
 							// unit clause - only optimize literal
-							opt.lits.push_back(WeightLiteral(~currentClause.back(), w));
+							Literal x = currentClause.back();
+							opt.lits.push_back(WeightLiteral(~x, w));
+							flags[x.var()] |= keep;
 							sat = true;
 						}
 					}
@@ -556,11 +487,14 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 	if (p) {
 		ctx.satPrepro.reset(p);
 	}
-	if (assertPure) {
+	if (pure == PureLitMode_t::assert_pure_auto && !opt.lits.empty()) {
+		pure = PureLitMode_t::assert_pure_no;
+	}
+	if (pure == PureLitMode_t::assert_pure_yes) {
 		for (int i = 1; ret && i <= lastVar && ret; ++i) {
-			uint8 d = (flags[i]>>2);
-			if      (d == 0)  { ret = ctx.addUnary(negLit(i)); }
-			else if (d != 3)  { ret = ctx.addUnary(Literal(i, d != 1)); }
+			if ((flags[i] & keep) != keep) {
+				ret = ctx.addUnary(Literal(i, ((flags[i]>>2) & 1u) != 1));
+			}
 		}
 	}
 	for (int v = lastVar+1; v <= maxVar; ++v) {
@@ -672,7 +606,7 @@ void OPBParser::parseHeader() {
 			if (!in.parseInt64(sum))          { ERROR("Sum of soft constraints cost expected\n"); }
 		}
 	}
-	if (!match(in, '\n', true))           { ERROR("Unrecognized characters in problem line\n"); }
+	if (!matchEol(in, true))              { ERROR("Unrecognized characters in problem line\n"); }
 	
 	// Init solver
 	maxVar_  = numVars_ + soft_ + products_;
@@ -703,7 +637,7 @@ void OPBParser::parseOptObjective() {
 		if (!in.parseInt64(obj_.rhs))  { ERROR("Top cost expected!\n"); }
 		if (obj_.rhs < 0)              { ERROR("Top cost must be positive!\n"); }
 		if (!match(in, ';', true))     { ERROR("Semicolon missing after constraint\n"); }
-		skipAllWhite(in);
+		in.skipWhite();
 	}
 }
 
@@ -732,7 +666,7 @@ void OPBParser::parseConstraint() {
 	int coeff;
 	if (!in.parseInt(coeff))   { ERROR("Missing coefficient on rhs of constraint\n"); }
 	if (!match(in, ';', true)) { ERROR("Semicolon missing after constraint\n"); }
-	skipAllWhite(in);
+	in.skipWhite();
 	if (x == 0 || lhs_.size() > 1 || lhs_[0].second != coeff+adjust_) {
 		addConstraint(relOp, coeff+adjust_, x);
 	}
@@ -762,7 +696,7 @@ void OPBParser::parseSum() {
 		if (*in == '>' || *in == '=') break;
 		in.skipWhite();
 	}
-	skipAllWhite(in);
+	in.skipWhite();
 }
 
 // <term>::=<variablename>
@@ -903,8 +837,8 @@ bool parseLparse(std::istream& prg, ProgramBuilder& api) {
 	return reader.parse(prg, api);
 }
 
-bool parseDimacs(std::istream& prg, SharedContext& ctx, bool assertPure, ObjectiveFunction& o, bool maxSat) {
-	return parseDimacsImpl(prg, ctx, assertPure, o, maxSat);
+bool parseDimacs(std::istream& prg, SharedContext& ctx, PureLitMode pm, ObjectiveFunction& o, bool maxSat) {
+	return parseDimacsImpl(prg, ctx, pm, o, maxSat);
 }
 
 bool parseOPB(std::istream& prg, SharedContext& ctx, ObjectiveFunction& objective) {
@@ -939,9 +873,10 @@ bool StreamInput::read(ApiPtr api, uint32 properties) {
 	}
 	else if (format_ == Input::DIMACS) {
 		func_.adjust = 0; func_.rhs = -1;
-		bool maxSat  = (properties & AS_MAX_SAT) != 0;
-		bool pure    = (properties & PRESERVE_MODELS) == 0 && (!maxSat || (properties & PRESERVE_MODELS_ON_MIN) == 0);
-		return parseDimacsImpl(prg_, *api.ctx, pure, func_, maxSat);
+		PureLitMode p= PureLitMode_t::assert_pure_yes;
+		if      ((properties & PRESERVE_MODELS) != 0)        { p = PureLitMode_t::assert_pure_no;   }
+		else if ((properties & PRESERVE_MODELS_ON_MIN) != 0) { p = PureLitMode_t::assert_pure_auto; }
+		return parseDimacsImpl(prg_, *api.ctx, p, func_, (properties & AS_MAX_SAT) != 0);
 	}
 	else {
 		return parseOPB(prg_, *api.ctx, func_);
