@@ -1365,7 +1365,7 @@ Solver::DBInfo Solver::reduceSortInPlace(uint32 maxR, const CmpScore& sc, bool p
 			*nEnd++ = c;
 		}
 	}
-	res.size = std::distance(learnts_.begin(), nEnd);
+	res.size = static_cast<uint32>(std::distance(learnts_.begin(), nEnd));
 	return res;	
 }
 
