@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006-2011, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -21,6 +21,36 @@
 #include <clasp/solver.h>
 namespace Clasp {
 /////////////////////////////////////////////////////////////////////////////////////////
+// SolveStats
+/////////////////////////////////////////////////////////////////////////////////////////
+void SolveStats::enableStats(uint32 level) {
+	if (level > 1 && !jumps)  { jumps  = new JumpStats();   }
+}
+void SolveStats::enableStats(const SolveStats& o) {
+	if (o.jumps    && !jumps)   { jumps   = new JumpStats(); }
+	if (o.parallel && !parallel){ parallel= new ParallelStats(); }
+}
+void SolveStats::enableQueue(uint32 size) { 
+	if (queue && queue->maxSize()!=size) { queue->destroy(); queue = 0; }
+	if (!queue)                          { queue = SumQueue::create(size); }
+}
+void SolveStats::reset() {
+	CoreStats::reset();
+	if (queue)   queue->reset();
+	if (jumps)   jumps->reset();
+	if (parallel)parallel->reset();
+}
+void SolveStats::accu(const SolveStats& o) {
+	CoreStats::accu(o);
+	if (jumps    && o.jumps)    jumps->accu(*o.jumps);
+	if (parallel && o.parallel) parallel->accu(*o.parallel);
+}
+void SolveStats::swapStats(SolveStats& o) {
+	std::swap(static_cast<CoreStats&>(*this), static_cast<CoreStats&>(o));
+	std::swap(jumps, o.jumps);
+	std::swap(parallel, o.parallel);
+}
+/////////////////////////////////////////////////////////////////////////////////////////
 // ClauseHead
 /////////////////////////////////////////////////////////////////////////////////////////
 ClauseHead::Info::Info(const Clasp::ClauseInfo& init) {
@@ -31,8 +61,8 @@ ClauseHead::Info::Info(const Clasp::ClauseInfo& init) {
 }
 
 ClauseHead::ClauseHead(const ClauseInfo& init) : info_(init){
-	static_assert(sizeof(ClauseHead)<=32);
-	head_[2]         = negLit(0);
+	static_assert(sizeof(ClauseHead)<=32, "Unsupported Alignment");
+	head_[2]  = negLit(0);
 }
 
 void ClauseHead::attach(Solver& s) {
@@ -56,27 +86,21 @@ bool ClauseHead::satisfied(const Solver& s) {
 }
 
 bool ClauseHead::toImplication(Solver& s) {
-	ConstraintType t = ClauseHead::type();
-	uint32 newSize   = isSentinel(head_[1]) ? 1 : 2 + (!s.isFalse(head_[2]) || s.level(head_[2].var()) > 0);
-	bool   upBTIG    = s.sharedContext()->updateShortImplications() && (t != Constraint_t::static_constraint || s.sharedContext()->unique());
-	bool   locked    = ClauseHead::locked(s) && s.decisionLevel() > 0;
-	if (newSize == 1 || (!tagged() && !locked && upBTIG)) {
-		uint32 bin = 0, tern = 0; uint64 learn = 0, lits = 0;
-		if (t != Constraint_t::static_constraint) {
-			learn = s.stats.learnts[t-1], lits = s.stats.lits[t-1];
-		}
-		assert(!tagged());
-		if      (newSize == 3) { tern = s.stats.ternary; s.addTernary(head_[0], head_[1], head_[2], t); s.stats.ternary = tern; }
-		else if (newSize == 2) { bin  = s.stats.binary;  s.addBinary(head_[0], head_[1], t); s.stats.binary = bin; }
-		else                   { s.addUnary(head_[0], t); }
-		detach(s);
-		if (t != Constraint_t::static_constraint) {
-			s.stats.learnts[t-1] = learn, s.stats.lits[t-1] = lits;
-		}
-		return true;
-	}
-	// clause can't be transformed at the moment
-	return false;
+	ConstraintType t  = ClauseHead::type();
+	uint32 sz         = isSentinel(head_[1]) ? 1 : 2 + (!s.isFalse(head_[2]) || s.level(head_[2].var()) > 0);
+	bool   implicit   = s.sharedContext()->allowImplicit(t);
+	bool   locked     = ClauseHead::locked(s) && s.decisionLevel() > 0;
+	if ((locked || !implicit || tagged()) && sz > 1) { return false; }
+	// transform to implication but keep statistics
+	uint32 i          = t-learnt();
+	uint32 bin        = s.stats.binary,     tern = s.stats.ternary;
+	uint64 learnt     = s.stats.learnts[i], lits = s.stats.lits[i];
+	if (sz > 1) { s.addShort(head_,sz, ClauseInfo(t).setLbd(2)); }
+	else        { s.addUnary(head_[0], t); }
+	s.stats.binary    = bin;    s.stats.ternary  = tern;
+	s.stats.learnts[i]= learnt; s.stats.lits[i]  = lits;
+	detach(s);
+	return true;
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // SmallClauseAlloc
