@@ -69,28 +69,26 @@ namespace Clasp {
 //   until the source condition holds again.
 // - If the condition cannot be restored, the body is marked as invalid source.
 
-DefaultUnfoundedCheck::DefaultUnfoundedCheck(ReasonStrategy r)
+DefaultUnfoundedCheck::DefaultUnfoundedCheck()
 	: solver_(0) 
 	, graph_(0)
 	, reasons_(0)
-	, activeClause_(0)
-	, strategy_(r) {
+	, strategy_(common_reason) {
 }
 DefaultUnfoundedCheck::~DefaultUnfoundedCheck() { 
 	for (ExtVec::size_type i = 0; i != extended_.size(); ++i) {
 		::operator delete(extended_[i]);
 	}
 	delete [] reasons_;
-	delete activeClause_;
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 // DefaultUnfoundedCheck - Initialization
 /////////////////////////////////////////////////////////////////////////////////////////
 void DefaultUnfoundedCheck::attachTo(Solver& s, DependencyGraph* graph) {
 	assert(solver_ == 0 && graph_ == 0);
-	solver_ = &s;
-	graph_  = graph;
-	activeClause_ = new ClauseCreator(solver_);
+	solver_  = &s;
+	graph_   = graph;
+	strategy_= static_cast<ReasonStrategy>(s.strategies().loopRep);
 	s.addPost(this);
 }
 
@@ -127,6 +125,10 @@ bool DefaultUnfoundedCheck::init(Solver&) {
 			addExtWatch(~a.lit, NodeId(i));
 		}
 	}
+	if (strategy_ == only_reason) {
+		delete [] reasons_;
+		reasons_ = new LitVec[solver_->numVars()];
+	}
 	return true;
 }
 
@@ -229,9 +231,9 @@ Constraint::PropResult DefaultUnfoundedCheck::propagate(Solver& s, Literal, uint
 
 void DefaultUnfoundedCheck::reason(Solver&, Literal p, LitVec& r) {
 	LitVec::const_iterator it, end;
-	if (!activeClause_->empty() && (*activeClause_)[0] == p) {
-		it  = activeClause_->lits().begin()+1;
-		end = activeClause_->lits().end();
+	if (!activeClause_.empty() && activeClause_[0] == p) {
+		it  = activeClause_.begin()+1;
+		end = activeClause_.end();
 	}
 	else {
 		assert(strategy_ == only_reason && reasons_);
@@ -259,7 +261,7 @@ void DefaultUnfoundedCheck::reset() {
 		sourceQ_.pop_back();
 	}
 	invalid_.clear();
-	activeClause_->clear();
+	activeClause_.clear();
 }
 
 bool DefaultUnfoundedCheck::propagateFixpoint(Solver&) {
@@ -492,7 +494,7 @@ void DefaultUnfoundedCheck::addUnsourced(const BodyNodeP& n) {
 
 // asserts all atoms of the unfounded set, then propagates
 bool DefaultUnfoundedCheck::assertSet() {
-	activeClause_->clear();
+	activeClause_.clear();
 	while (findUnfoundedSet()) {
 		while (!unfounded_.empty() && solver_->force(~graph_->getAtomNode(unfounded_.front()).lit, 0)) {
 			dequeueUnfounded();
@@ -510,81 +512,63 @@ bool DefaultUnfoundedCheck::assertSet() {
 // - removes the atom from U
 bool DefaultUnfoundedCheck::assertAtom() {
 	while(findUnfoundedSet()) {
-		activeClause_->clear();
+		activeClause_.clear();
 		while (!unfounded_.empty()) {
-			Literal l = graph_->getAtomNode(unfounded_.front()).lit;
-			if (!solver_->isFalse(l) && !assertAtom(l)) {
-				return false;
+			Literal a = graph_->getAtomNode(unfounded_.front()).lit;
+			if (!solver_->isFalse(a) && !(assertAtom(a) && solver_->propagateUntil(this))) {
+				break;
 			}
+			assert(solver_->isFalse(a));
 			dequeueUnfounded();
 		}
-		unfounded_.clear();
 		if (!loopAtoms_.empty()) {
 			createLoopFormula();
 		}
+		if (!unfounded_.empty()) { 
+			return false; 
+		}
+		unfounded_.clear(); // clear underlying vector
 	}
 	return true;
 }
 
 // asserts an unfounded atom using the selected reason strategy
 bool DefaultUnfoundedCheck::assertAtom(Literal a) {
-	if (strategy_ == distinct_reason || activeClause_->empty()) {
-		// first atom of unfounded set or distinct reason for each atom requested.
-		activeClause_->startAsserting(Constraint_t::learnt_loop, ~a);
+	if (solver_->isTrue(a) || strategy_ == distinct_reason || activeClause_.empty()) {
+		// Conflict, first atom of unfounded set, or distinct reason for each atom requested -
+		// compute reason for a being unfounded.
+		// We must flush any not yet created loop formula here - the
+		// atoms in loopAtoms_ depend on the current reason which is about to be replaced. 
+		if (!loopAtoms_.empty()) { 
+			createLoopFormula(); 
+		}
+		activeClause_.assign(1, ~a);
 		computeReason();
-		if (solver_->isTrue(a) || strategy_ == only_reason || (strategy_ == shared_reason && activeClause_->size() > 3)) {
-			if (!solver_->force(~a, this)) return false;
-			if (strategy_ == only_reason) {
-				if (reasons_ == 0) reasons_ = new LitVec[solver_->numVars()];
-				reasons_[a.var()-1].assign(activeClause_->lits().begin()+1, activeClause_->lits().end());
-			}
-			else {
-				loopAtoms_.push_back(~a);
-			}
-		}
-		else {
-			activeClause_->end(); // learn nogood and assert ~a
-		}
 	}
-	// subsequent atoms
-	else if (solver_->isTrue(a)) {
-		if (!loopAtoms_.empty()) {
-			createLoopFormula();
-		}
-		(*activeClause_)[0] = ~a;
-		return solver_->force(a, this);
+	activeClause_[0] = ~a;
+	bool noClause = solver_->isTrue(a) || strategy_ == only_reason || (strategy_ == shared_reason && activeClause_.size() > 3);
+	if (noClause) {
+		if (!solver_->force(~a, this)){ return false; }
+		if (strategy_ == only_reason) { reasons_[a.var()-1].assign(activeClause_.begin()+1, activeClause_.end()); }
+		else                          { loopAtoms_.push_back(~a); }
+		return true;
 	}
-	else if (strategy_ == only_reason || (strategy_ == shared_reason && activeClause_->size() > 3)) {
-		solver_->force(~a, this);
-		if (strategy_ == only_reason) {
-			reasons_[a.var()-1].assign(activeClause_->lits().begin()+1, activeClause_->lits().end());
-		}
-		else {
-			loopAtoms_.push_back(~a);
-		}
+	else { // learn nogood and assert ~a
+		return ClauseCreator::create(*solver_, activeClause_, info_).ok();
 	}
-	else {
-		(*activeClause_)[0] = ~a;
-		activeClause_->end();
-	}
-	bool ok = solver_->propagateUntil(this);
-	if (!ok && !loopAtoms_.empty()) {
-		createLoopFormula();
-	}
-	return ok;
 }
-
 void DefaultUnfoundedCheck::createLoopFormula() {
-	assert(activeClause_->size() > 3);
+	assert(activeClause_.size() > 3);
 	if (loopAtoms_.size() == 1) {
-		(*activeClause_)[0] = loopAtoms_[0];
-		Constraint* ante = activeClause_->end().local;
+		activeClause_[0] = loopAtoms_[0];
+		Constraint* ante = ClauseCreator::create(*solver_, activeClause_, info_).local;
 		assert(ante != 0 && solver_->isTrue(loopAtoms_[0]) && solver_->reason(loopAtoms_[0]) == this);
 		solver_->setReason(loopAtoms_[0], ante);
 	}
 	else {
-		LoopFormula* lf = LoopFormula::newLoopFormula(*solver_, &(*activeClause_)[1], (uint32)activeClause_->size() - 1, (uint32)activeClause_->sw()-1, (uint32)loopAtoms_.size(), activeClause_->info()); 
-		solver_->addLearnt(lf, lf->size());
+		Activity act(info_.activity(), info_.lbd());
+		LoopFormula* lf = LoopFormula::newLoopFormula(*solver_, &activeClause_[1], (uint32)activeClause_.size() - 1, (uint32)0, (uint32)loopAtoms_.size(), act); 
+		solver_->addLearnt(lf, lf->size(), Constraint_t::learnt_loop);
 		for (VarVec::size_type i = 0; i < loopAtoms_.size(); ++i) {
 			assert(solver_->isTrue(loopAtoms_[i]) && solver_->reason(loopAtoms_[i]) == this);
 			solver_->setReason(loopAtoms_[i], lf);
@@ -607,41 +591,20 @@ void DefaultUnfoundedCheck::computeReason() {
 			}
 		}
 	}
-	assert( activeClause_->implicationLevel() == solver_->decisionLevel()
-		&& "Loop nogood must contain a literal from current DL!");
-	LitVec::size_type i    = 1;
-	uint32 lbd = 1, actLbd = 1;
-	Var    v   = (*activeClause_)[0].var();
-	if (solver_->value(v) != value_free && !solver_->seen(v)) {
-		i   = 0;
-		lbd = actLbd = 0;
-		solver_->markLevel(solver_->level((*activeClause_)[0].var()));
-	}
-	for (uint32 vLev; i != activeClause_->size(); ++i) {
-		v    = (*activeClause_)[i].var();
-		vLev = solver_->level(v);
-		solver_->clearSeen( v );
-		if (solver_->hasLevel(vLev)) {
-			++lbd;
-			actLbd += (vLev > solver_->rootLevel());
-			solver_->unmarkLevel(vLev);
-		}
-	}
-	for (i = 0; i != pickedExt_.size(); ++i) {
-		bodies_[pickedExt_[i]].picked = 0;
-	}
+	for (VarVec::size_type i = 0; i != pickedExt_.size(); ++i) { bodies_[pickedExt_[i]].picked = 0; }
 	pickedExt_.clear();
-	activeClause_->setActivity(static_cast<uint32>(1+solver_->stats.restarts));
-	activeClause_->setLbd(lbd, actLbd);
-	double ratio = activeClause_->size()/double(solver_->decisionLevel()+1);
-	if (ratio > 10 && activeClause_->size() > 100 && !solver_->isFalse((*activeClause_)[0])) {
-		Literal a = (*activeClause_)[0];
-		activeClause_->startAsserting(Constraint_t::learnt_loop, a);
-		for (uint32 d = 1; d <= solver_->decisionLevel(); ++d) {
-			activeClause_->add(~solver_->decision(d));
+	info_     = ClauseInfo(Constraint_t::learnt_loop);
+	uint32 dl = solver_->finalizeConflictClause(activeClause_, info_);
+	assert(dl == solver_->decisionLevel() && "Loop nogood must contain a literal from current DL!");
+	double ratio = activeClause_.size()/double(solver_->decisionLevel()+1);
+	if (ratio > 10 && activeClause_.size() > 100 && !solver_->isFalse(activeClause_[0])) {
+		activeClause_.resize(1);
+		// replace reason with current decision sequence
+		// start with last decision so that watches are correctly initialized
+		for (uint32 x = dl; x; --x) {
+			activeClause_.push_back(~solver_->decision(x));
 		}
-		activeClause_->setLbd(solver_->decisionLevel(), solver_->decisionLevel() - solver_->rootLevel());
-	}
+ 	}
 }
 
 // check if n is part of the reason for the current unfounded set
@@ -700,7 +663,10 @@ void DefaultUnfoundedCheck::addReasonLit(Literal p) {
 	if (!solver_->seen(p)) {
 		solver_->markSeen(p);
 		solver_->markLevel(solver_->level(p.var()));
-		activeClause_->add(p);
+		activeClause_.push_back(p);
+		if (solver_->level(p.var()) > solver_->level(activeClause_[1].var())) {
+			std::swap(activeClause_[1], activeClause_.back());
+		}
 	}
 }
 
