@@ -118,7 +118,7 @@ public:
 	}
 
 	void testCreatorAssertsFirstLit() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(~b);
 		solver().assume(~c);
 		solver().assume(~d);
@@ -132,7 +132,7 @@ public:
 	}
 
 	void testCreatorInitsWatches() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(~b);
 		solver().assume(~c);
 		solver().assume(~d);
@@ -153,9 +153,9 @@ public:
 			std::vector<LitVec::size_type> clSizes_;
 			std::vector<ConstraintType> clTypes_;
 		}*heu = new FakeHeu;
-		solver().strategies().heuristic.reset(heu);
+		solver().setHeuristic(-1, heu);
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.start().add(a).add(b).add(c).add(d).end());
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(a);
 		solver().assume(b);
 		solver().propagate();
@@ -191,7 +191,7 @@ public:
 		CPPUNIT_ASSERT(0 == ctx.numConstraints());
 	}
 	void testCreatorSimplifyMovesWatch() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(a); solver().propagate();
 		solver().assume(b); solver().propagate();
 		creator.start(Constraint_t::learnt_loop);
@@ -202,7 +202,7 @@ public:
 	}
 
 	void testCreateNonAssertingLearntClause() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(a); solver().propagate();
 		solver().assume(b); solver().propagate();
 		creator.start(Constraint_t::learnt_loop);
@@ -223,7 +223,7 @@ public:
 		CPPUNIT_ASSERT(ctx.numLearntShort() == 1);
 	}
 	void testCreateLearntClauseConflict() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
 		solver().assume(c); solver().propagate();										// level 2
 		solver().assume(d); solver().propagate();										// level 3
@@ -251,7 +251,7 @@ public:
 	}
 
 	void testCreateNonAssertingLearntClauseAsserting() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
 		solver().assume(c); solver().propagate();										// level 2
 		creator.start(Constraint_t::learnt_loop);
@@ -287,7 +287,7 @@ public:
 		CPPUNIT_ASSERT(ClauseCreator::status(solver(), &creator.lits()[0], &creator.lits()[0] + creator.size()) == ClauseCreator::status_sat);
 
 		ClauseCreator::Result r = creator.end();
-		CPPUNIT_ASSERT_EQUAL(true, r.ok);
+		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 		CPPUNIT_ASSERT(solver().decisionLevel() == 3);
 
 		creator.start(Constraint_t::learnt_other).add(a).add(~c).end();
@@ -324,7 +324,7 @@ public:
 		solver().pushRootLevel(solver().decisionLevel());
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT_EQUAL(false, r.ok);
+		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT_EQUAL(true, solver().hasConflict());
 		CPPUNIT_ASSERT_EQUAL(false, solver().clearAssumptions());
 	}
@@ -375,7 +375,7 @@ public:
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b); cl.push_back(~d);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT(!r.ok);
+		CPPUNIT_ASSERT(!r.ok());
 		CPPUNIT_ASSERT(r.local != 0);
 		CPPUNIT_ASSERT(solver().hasConflict());
 	}
@@ -420,7 +420,7 @@ public:
 		solver().pushRootLevel(solver().decisionLevel());
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_explicit);
-		CPPUNIT_ASSERT_EQUAL(false, r.ok);
+		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT_EQUAL(uint32(1), solver().numLearntConstraints());
 	}
 	
@@ -432,7 +432,7 @@ public:
 		solver().assume(~d) && solver().propagate();
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT_EQUAL(true, r.ok);
+		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 		CPPUNIT_ASSERT(3u == solver().numAssignedVars());
 	}
 	void testIntegrateSATBug2() {
@@ -444,7 +444,7 @@ public:
 		solver().force(c,0) && solver().propagate();
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT_EQUAL(true, r.ok);
+		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 	}
 
 	void testIntegrateSATBug3() {
@@ -458,7 +458,7 @@ public:
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_sat);
 		
-		CPPUNIT_ASSERT_EQUAL(true, r.ok);
+		CPPUNIT_ASSERT_EQUAL(true, r.ok());
 		CPPUNIT_ASSERT(solver().decisionLevel() == 2);
 		CPPUNIT_ASSERT(solver().isTrue(c));
 	}
@@ -490,7 +490,7 @@ public:
 		solver().force(~b,0)&& solver().propagate();
 		CPPUNIT_ASSERT(ClauseCreator::status(solver(), p->begin(), p->end()) == ClauseCreator::status_conflicting);
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_conflict);
-		CPPUNIT_ASSERT(r.ok == false);
+		CPPUNIT_ASSERT(r.ok() == false);
 		CPPUNIT_ASSERT(r.local == 0);
 	}
 	void testIntegrateSimplify() {
@@ -501,7 +501,7 @@ public:
 		solver().force(~d, 0) && solver().propagate();
 		solver().assume(~a)   && solver().propagate();
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_no_add);
-		CPPUNIT_ASSERT(r.ok);
+		CPPUNIT_ASSERT(r.ok());
 		CPPUNIT_ASSERT(r.local != 0);
 		cl.clear();
 		r.local->toLits(cl);
@@ -516,7 +516,7 @@ public:
 		do {
 			SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 			ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, ClauseCreator::clause_not_sat);
-			CPPUNIT_ASSERT_EQUAL(true, r.ok);
+			CPPUNIT_ASSERT_EQUAL(true, r.ok());
 			CPPUNIT_ASSERT(solver().numAssignedVars() == 3);
 		} while (std::next_permutation(cl.begin(), cl.end()));
 		CPPUNIT_ASSERT(solver().numLearntConstraints() == 0);
@@ -529,7 +529,7 @@ public:
 		cl.push_back(a);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT_EQUAL(false, r.ok);
+		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT(0 == r.local);
 		CPPUNIT_ASSERT_EQUAL(uint32(0), solver().decisionLevel());
 	}
@@ -545,7 +545,7 @@ public:
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT_EQUAL(false, r.ok);
+		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		solver().backtrack();
 		CPPUNIT_ASSERT(solver().isTrue(~c));
 		CPPUNIT_ASSERT_EQUAL(uint32(2), solver().decisionLevel());
@@ -564,7 +564,7 @@ public:
 		cl.push_back(~a); cl.push_back(~c); cl.push_back(~b);
 		SharedLiterals* p(SharedLiterals::newShareable(cl, Constraint_t::learnt_other));
 		ClauseCreator::Result r = ClauseCreator::integrate(solver(), p, 0);
-		CPPUNIT_ASSERT_EQUAL(false, r.ok);
+		CPPUNIT_ASSERT_EQUAL(false, r.ok());
 		CPPUNIT_ASSERT_EQUAL(true, solver().resolveConflict());
 		CPPUNIT_ASSERT_EQUAL(uint32(1), solver().decisionLevel());
 	}
@@ -572,7 +572,7 @@ public:
 
 	void testFactsAreRemovedFromLearnt() {
 		ctx.addUnary(a);
-		ctx.endInit(1);
+		ctx.endInit();
 		solver().assume(~b) && solver().propagate();
 		creator.start(Constraint_t::learnt_conflict);
 		creator.add(b).add(c).add(~a).end();
