@@ -113,15 +113,15 @@ public:
 		ClauseHead* cl1;
 		solver->add(cl1 = createClause(2,2));
 		CPPUNIT_ASSERT_EQUAL(2, countWatches(*solver, cl1, clLits) );
-		ClauseHead* cl2 = Clause::newLearntClause(*solver, clLits, ClauseInfo().setType(Constraint_t::learnt_conflict));
+		ClauseHead* cl2 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict));
 		solver->add(cl2);
 		CPPUNIT_ASSERT_EQUAL(2,  countWatches(*solver, cl2, clLits));
 	}
 
 	void testClauseTypes() {
 		Clause* cl1 = createClause(2, 2);
-		LearntConstraint* cl2 = Clause::newLearntClause(*solver, clLits, ClauseInfo().setType(Constraint_t::learnt_conflict));
-		LearntConstraint* cl3 = Clause::newLearntClause(*solver, clLits, ClauseInfo().setType(Constraint_t::learnt_loop));
+		LearntConstraint* cl2 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict));
+		LearntConstraint* cl3 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_loop));
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::static_constraint, cl1->type());
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_conflict, cl2->type());
 		CPPUNIT_ASSERT_EQUAL(Constraint_t::learnt_loop, cl3->type());
@@ -137,17 +137,17 @@ public:
 		lit.push_back(posLit(3));
 		lit.push_back(posLit(4));
 		uint32 exp = 258;
-		ClauseHead* cl1 = Clause::newLearntClause(*solver, lit, ClauseInfo().setType(Constraint_t::learnt_conflict).setActivity(exp));
-		ClauseHead* cl2 = Clause::newLearntClause(*solver, lit, ClauseInfo().setType(Constraint_t::learnt_loop).setActivity(exp));
+		ClauseHead* cl1 = Clause::newLearntClause(*solver, lit, ClauseInfo(Constraint_t::learnt_conflict).setActivity(exp));
+		ClauseHead* cl2 = Clause::newLearntClause(*solver, lit, ClauseInfo(Constraint_t::learnt_loop).setActivity(exp));
 		solver->add(cl1);
 		solver->add(cl2);
 		while ( exp != 0 ) {
-			CPPUNIT_ASSERT(cl1->activity().score() == cl2->activity().score() && cl1->activity().score() == exp);
+			CPPUNIT_ASSERT(cl1->activity().activity() == cl2->activity().activity() && cl1->activity().activity() == exp);
 			exp >>= 1;
 			cl1->decreaseActivity();
 			cl2->decreaseActivity();
 		}
-		CPPUNIT_ASSERT(cl1->activity().score() == cl2->activity().score() && cl1->activity().score() == exp);
+		CPPUNIT_ASSERT(cl1->activity().activity() == cl2->activity().activity() && cl1->activity().activity() == exp);
 	}
 
 	void testPropGenericClause() {
@@ -209,7 +209,7 @@ public:
 		for (int i = 0; i != 100; ++i) {
 			SharedContext cc;
 			solver = cc.master();
-			solver->strategies().randomWatches = true;
+			solver->strategies().initWatches = SolverStrategies::watch_rand;
 			for (int j = 0; j < 12; ++j) { cc.addVar(Var_t::atom_var); }
 			cc.startAddConstraints(1);
 			Clause* c;
@@ -223,8 +223,8 @@ public:
 		clLits.push_back(posLit(2));
 		clLits.push_back(posLit(3));
 		clLits.push_back(posLit(4));
-		ctx.endInit(1);
-		LearntConstraint* cl1 = Clause::newLearntClause(*solver, clLits, ClauseInfo().setType(Constraint_t::learnt_conflict));
+		ctx.endInit();
+		ClauseHead* cl1 = Clause::newLearntClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict));
 		solver->addLearnt(cl1, (uint32)clLits.size());
 		solver->assume(~clLits[0]);
 		solver->propagate();
@@ -234,10 +234,10 @@ public:
 		solver->propagate();
 		
 		CPPUNIT_ASSERT_EQUAL(true, solver->isTrue( clLits[3] ) );
-		uint32 a = cl1->activity().score();
+		uint32 a = cl1->activity().activity();
 		LitVec r;
 		solver->reason(clLits[3], r);
-		CPPUNIT_ASSERT_EQUAL(a+1, cl1->activity().score());
+		CPPUNIT_ASSERT_EQUAL(a+1, cl1->activity().activity());
 	}
 
 	void testSimplifySAT() {
@@ -365,7 +365,7 @@ public:
 		clLits.clear(); 
 		clLits.push_back(b);
 		clLits.push_back(~a);
-		ClauseInfo extra; extra.tag().setType(Constraint_t::learnt_conflict);
+		ClauseInfo extra(Constraint_t::learnt_conflict); extra.setTagged(true);
 		ClauseHead* c = ClauseCreator::create(*solver, clLits, extra).local;
 		CPPUNIT_ASSERT(c->size() == 2);
 		CPPUNIT_ASSERT(solver->isTrue(b) && solver->reason(b).constraint() == c);
@@ -383,7 +383,7 @@ public:
 
 	void testStrengthenContracted() {
 		solver->strategies().compress = 4;
-		ctx.endInit(1);
+		ctx.endInit();
 		ClauseCreator creator(solver);
 		creator.startAsserting(Constraint_t::learnt_conflict, a1);
 		for (uint32 i = 2; i <= 12; ++i) {
@@ -413,19 +413,20 @@ public:
 		CPPUNIT_ASSERT_EQUAL(false, c->strengthen(*solver, posLit(7), false).second);
 		CPPUNIT_ASSERT_EQUAL(uint32(3), c->size());
 		CPPUNIT_ASSERT_EQUAL(true, c->strengthen(*solver, posLit(11)).second);
+		CPPUNIT_ASSERT_EQUAL(sizeof(Clause) + (9*sizeof(Literal)), ((Clause*)c)->computeAllocSize());
 	}
 
 	void testStrengthenBug() {
 		solver->strategies().compress = 6;
-		ctx.endInit(1);
+		ctx.endInit();
 		LitVec clause;
 		clause.push_back(a1);
 		for (uint32 i = 2; i <= 6; ++i) {
 			solver->assume(negLit(8-i)) && solver->propagate();
 			clause.push_back(posLit(i));
 		}
-		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(), 5, true);
-		solver->add(c);
+		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(Constraint_t::learnt_conflict), 5, true);
+		solver->addLearnt(c, 5);
 		uint32 si = c->size();
 		CPPUNIT_ASSERT(si == 5);
 		c->strengthen(*solver, posLit(4));
@@ -439,15 +440,15 @@ public:
 
 	void testStrengthenContractedNoExtend() {
 		solver->strategies().compress = 4;
-		ctx.endInit(1);
+		ctx.endInit();
 		LitVec clause;
 		clause.push_back(a1);
 		for (uint32 i = 2; i <= 6; ++i) {
 			solver->assume(negLit(8-i)) && solver->propagate();
 			clause.push_back(posLit(i));
 		}
-		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(), 4, false);
-		solver->add(c);
+		ClauseHead* c = Clause::newContractedClause(*solver, clause, ClauseInfo(Constraint_t::learnt_conflict), 4, false);
+		solver->addLearnt(c, 4);
 		CPPUNIT_ASSERT(c->size() == 4);
 		c->strengthen(*solver, posLit(2));
 		CPPUNIT_ASSERT(c->size() == 4);
@@ -539,28 +540,32 @@ public:
 	}
 
 	void testClauseSatisfied() {
+		ConstraintType t = Constraint_t::learnt_conflict;
+		TypeSet ts; ts.addSet(t);
 		Clause* c;
-		solver->add(c = createClause(2, 2));
+		solver->addLearnt(c = createClause(2, 2, t), 4);
 		LitVec free;
-		CPPUNIT_ASSERT_EQUAL(false, c->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(t), c->isOpen(*solver, ts, free));
 		CPPUNIT_ASSERT_EQUAL(LitVec::size_type(4), free.size());
 
 		solver->assume( clLits[2] );
 		solver->propagate();
 		free.clear();
-		CPPUNIT_ASSERT_EQUAL(true, c->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(0), c->isOpen(*solver, ts, free));
 		solver->undoUntil(0);
 		solver->assume( ~clLits[1] );
 		solver->assume( ~clLits[2] );
 		solver->propagate();
 		free.clear();
-		CPPUNIT_ASSERT_EQUAL(false, c->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(t), c->isOpen(*solver, ts, free));
 		CPPUNIT_ASSERT_EQUAL(LitVec::size_type(2), free.size());
+		ts.m = 0; ts.addSet(Constraint_t::learnt_loop);
+		CPPUNIT_ASSERT_EQUAL(uint32(0), c->isOpen(*solver, ts, free));
 	}
 	
 	void testContraction() {
 		solver->strategies().compress = 6;
-		ctx.endInit(1);
+		ctx.endInit();
 		ClauseCreator creator(solver);
 		creator.startAsserting(Constraint_t::learnt_conflict, a1);
 		for (uint32 i = 2; i <= 12; ++i) {
@@ -579,7 +584,7 @@ public:
 	}
 
 	void testNewContractedClause() {
-		ctx.endInit(1);
+		ctx.endInit();
 		// head
 		clLits.push_back(a1);
 		clLits.push_back(a2);
@@ -589,7 +594,7 @@ public:
 			// (false) tail
 			clLits.push_back(posLit(i));
 		}
-		ClauseHead* c = Clause::newContractedClause(*solver, clLits, ClauseInfo().setType(Constraint_t::learnt_conflict), 3, false);
+		ClauseHead* c = Clause::newContractedClause(*solver, clLits, ClauseInfo(Constraint_t::learnt_conflict), 3, false);
 		solver->addLearnt(c, static_cast<uint32>(clLits.size()));
 		CPPUNIT_ASSERT(c->size() < clLits.size());
 
@@ -629,7 +634,8 @@ public:
 
 	void testClone() {
 		Solver solver2;
-		ctx.endInit(2);
+		ctx.setSolvers(2);
+		ctx.endInit();
 		ctx.attach(solver2);
 		ClauseHead* c      = createClause(3, 3);
 		ClauseHead* clone  = (ClauseHead*)c->cloneAttach(solver2);
@@ -683,6 +689,7 @@ public:
 		solver->propagate();
 		CPPUNIT_ASSERT_EQUAL(true, lf->simplify(*solver));
 		CPPUNIT_ASSERT_EQUAL(false, solver->hasWatch(~b3, lf));
+		solver->reduceLearnts(1.0f);
 	}
 
 	void testSimplifyLFRemovesFalseBodies() {
@@ -904,25 +911,28 @@ public:
 
 	void testLoopFormulaSatisfied() {
 		LoopFormula* lf = lfTestInit();
+		ConstraintType t= Constraint_t::learnt_loop;
+		TypeSet ts, other; ts.addSet(t); other.addSet(Constraint_t::learnt_conflict);
 		LitVec free;
-		CPPUNIT_ASSERT_EQUAL(true, lf->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(0), lf->isOpen(*solver, ts, free));
 		solver->undoUntil(0);
 		free.clear();
-		CPPUNIT_ASSERT_EQUAL(false, lf->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(lf->type()), lf->isOpen(*solver, ts, free));
 		CPPUNIT_ASSERT_EQUAL(LitVec::size_type(6), free.size());
+		CPPUNIT_ASSERT_EQUAL(uint32(0), lf->isOpen(*solver, other, free));
 		solver->assume( a1 );
 		solver->assume( ~b2 );
 		solver->propagate();
 		free.clear();
-		CPPUNIT_ASSERT_EQUAL(false, lf->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(lf->type()), lf->isOpen(*solver, ts, free));
 		CPPUNIT_ASSERT_EQUAL(LitVec::size_type(4), free.size());
 		solver->assume( ~b1 );
 		solver->propagate();
-		CPPUNIT_ASSERT_EQUAL(true, lf->isSatisfied(*solver, free));
+		CPPUNIT_ASSERT_EQUAL(uint32(0), lf->isOpen(*solver, ts, free));
 	}
 
 	void testLoopFormulaBugEq() {
-		ctx.endInit(1);
+		ctx.endInit();
 		Literal body1 = b1; 
 		Literal body2 = b2;
 		Literal body3 = ~b3; // assume body3 is equivalent to some literal ~xy
@@ -931,7 +941,7 @@ public:
 		solver->assume(~body3);
 		solver->propagate();
 		Literal bodies[3] = {body1, body2, body3 };
-		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, &bodies[0], 3, 2, 1);
+		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, &bodies[0], 3, 2, 1, Activity(0,10));
 		solver->addLearnt(lf, lf->size());
 		solver->force(~a1, lf);
 		lf->addAtom(~a1, *solver);
@@ -1000,7 +1010,7 @@ private:
 		res+="]";
 		return res;
 	}
-	Clause* createClause(int pos, int neg) {
+	Clause* createClause(int pos, int neg, ConstraintType t = Constraint_t::static_constraint) {
 		clLits.clear();
 		int size = pos + neg;
 		LitVec lit(size);
@@ -1012,7 +1022,7 @@ private:
 			lit[i] = negLit(i+1);
 			clLits.push_back(lit[i]);
 		}
-		return (Clause*)Clause::newClause(*solver, lit);
+		return (Clause*)Clause::newClause(*solver, &lit[0], (uint32)lit.size(), ClauseInfo(t));
 	}
 
 	Clause* createRandomClause(int size) {
@@ -1021,13 +1031,13 @@ private:
 	}
 	
 	LoopFormula* lfTestInit() {
-		ctx.endInit(1);
+		ctx.endInit();
 		solver->assume(~b1);
 		solver->assume(~b2);
 		solver->assume(~b3);
 		solver->propagate();
 		Literal bodies[3] = {b1, b2, b3 };
-		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, &bodies[0], 3, 2, 3);
+		LoopFormula* lf = LoopFormula::newLoopFormula(*solver, &bodies[0], 3, 2, 3, Activity(0,10));
 		solver->addLearnt(lf, lf->size());
 		solver->force(~a1, lf);
 		solver->force(~a2, lf);
