@@ -841,8 +841,8 @@ public:
 		solver.assume(minAssume);
 		solver.pushRootLevel(1);
 		solver.propagate();
-		
-		setOptimum(solver, SharedMinimizeData::SumVec(1, 0), false);
+		SharedMinimizeData::SumVec opt(1, 0);
+		setOptimum(solver, opt, false);
 		CPPUNIT_ASSERT(solver.isFalse(a) && solver.reason(~a).constraint() == newMin);
 		CPPUNIT_ASSERT(solver.isFalse(b) && solver.reason(~b).constraint() == newMin);
 		CPPUNIT_ASSERT(solver.isFalse(c) && solver.reason(~c).constraint() == newMin);
