@@ -522,7 +522,8 @@ public:
 			.startRule(CHOICERULE).addHead(1).addHead(2).addHead(3).endRule()
 			.startRule().addHead(4).addToBody(1, true).addToBody(2, true).addToBody(3, true).endRule()
 		;
-		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit(2));
+		ctx.setSolvers(2);
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
 		CPPUNIT_ASSERT_EQUAL( uint32(6), ctx.numVars() );
 		CPPUNIT_ASSERT_EQUAL( uint32(6), ctx.numConstraints() );
 
@@ -532,7 +533,7 @@ public:
 		CPPUNIT_ASSERT_EQUAL( uint32(6), solver2.numVars() );
 		CPPUNIT_ASSERT_EQUAL( uint32(6), solver2.numConstraints() );
 
-		CPPUNIT_ASSERT(!ctx.unique());
+		CPPUNIT_ASSERT(ctx.isShared());
 	}
 
 	void testCloneShareSymbols() {
@@ -551,8 +552,8 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, builder.cloneProgram(ctx2) && ctx2.endInit());
 		CPPUNIT_ASSERT_EQUAL( uint32(6), ctx2.numVars() );
 		CPPUNIT_ASSERT_EQUAL( uint32(6), ctx2.numConstraints() );
-		CPPUNIT_ASSERT(ctx.unique());
-		CPPUNIT_ASSERT(ctx2.unique());
+		CPPUNIT_ASSERT(!ctx.isShared());
+		CPPUNIT_ASSERT(!ctx2.isShared());
 		CPPUNIT_ASSERT( &ctx.symTab() == &ctx2.symTab() );
 	}
 	void testCloneFull() {
@@ -571,8 +572,8 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, builder.cloneProgram(ctx2) && ctx2.endInit());
 		CPPUNIT_ASSERT_EQUAL( uint32(6), ctx2.numVars() );
 		CPPUNIT_ASSERT_EQUAL( uint32(6), ctx2.numConstraints() );
-		CPPUNIT_ASSERT(ctx.unique());
-		CPPUNIT_ASSERT(ctx2.unique());
+		CPPUNIT_ASSERT(!ctx.isShared());
+		CPPUNIT_ASSERT(!ctx2.isShared());
 		CPPUNIT_ASSERT( &ctx.symTab() != &ctx2.symTab() );
 	}
 
@@ -641,7 +642,7 @@ public:
 
 	void testAddClauses() {
 		ClauseObserver* o = new ClauseObserver;
-		ctx.master()->strategies().heuristic.reset(o);
+		ctx.master()->setHeuristic(7, o);
 		builder.startProgram(ctx)
 			.setAtomName(1, "a").setAtomName(2, "b").setAtomName(3, "c")
 			.startRule().addHead(1).addToBody(2, false).endRule()								// a :- not b.
