@@ -221,6 +221,7 @@ class RuleTransformTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testWeightSixFourExp);
 	CPPUNIT_TEST(testWeightSixFourQuad);
 	CPPUNIT_TEST(testWeightBug);
+	CPPUNIT_TEST(testDegeneratedWeightRule);
 
 	CPPUNIT_TEST(testStupidWeightBug);
 	CPPUNIT_TEST(testWeightBogusNormal);
@@ -512,6 +513,29 @@ public:
 		CPPUNIT_ASSERT(out.str().find(exp.str()) != std::string::npos);
 	}
 
+	void testDegeneratedWeightRule() {
+		prg.setExtendedRuleMode(ProgramBuilder::mode_transform_weight);
+		prg.startRule(CHOICERULE).addHead(2).addHead(3).addHead(4).endRule();
+		rule.setType(WEIGHTRULE);
+		// a :- 20 {b=18, c=18, d=18}
+		rule.addHead(1).addToBody(2, true,18).addToBody(3, true,18).addToBody(4, true,18);
+		rule.setBound(20);
+		RuleState rs;
+		rule.simplify(rs);
+		CPPUNIT_ASSERT(rule.type() == CONSTRAINTRULE);
+		PrgRuleTransform tm;
+		CPPUNIT_ASSERT_EQUAL(3u, tm.transformNoAux(prg, rule));
+		prg.endProgram();
+		std::ostringstream out;
+		prg.writeProgram(out);
+		std::stringstream exp;
+		exp 
+			<< "1 1 2 0 2 3 \n"  // a	:- b, c
+			<< "1 1 2 0 2 4 \n"  // a	:- b, d
+			<< "1 1 2 0 3 4 \n"; // a	:- c, d.	
+		CPPUNIT_ASSERT(out.str().find(exp.str()) != std::string::npos);
+	}
+
 	void testStupidWeightBug() {
 		prg.setAtomName(8, "h").setAtomName(9, "x");
 		prg.setExtendedRuleMode(ProgramBuilder::mode_transform_weight);
