@@ -201,6 +201,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL(0u, s.numLearntConstraints());
 		CPPUNIT_ASSERT_EQUAL(0u, s.decisionLevel());
 		CPPUNIT_ASSERT_EQUAL(0u, s.queueSize());
+
+		ctx.setFrozen(0, true);
+		CPPUNIT_ASSERT(ctx.stats().vars_frozen == 0);
 	}
 	void testVarNullIsSentinel() {
 		Literal p = posLit(0);
