@@ -522,10 +522,11 @@ public:
 	}
 
 	void testCloneWeight() {
+		ctx.setSolvers(2);
 		WeightLitVec lits = makeWeightLits();
 		CPPUNIT_ASSERT_EQUAL(true, WeightConstraint::newWeightConstraint(ctx, body, lits, 3));
 		solver().force(~a, 0);
-		ctx.endInit(2);
+		ctx.endInit();
 		Solver solver2;
 		ctx.attach(solver2);
 		CPPUNIT_ASSERT(solver2.numConstraints() == 1);
@@ -548,11 +549,12 @@ public:
 	}
 
 	void testCloneWeightShared() {
-		ctx.enableConstraintSharing();
+		ctx.setSolvers(2);
+		ctx.physicalSharing(SharedContext::share_problem);
 		WeightLitVec lits = makeWeightLits();
 		CPPUNIT_ASSERT_EQUAL(true, WeightConstraint::newWeightConstraint(ctx, body, lits, 3));
 		solver().force(~a, 0);
-		ctx.endInit(2);
+		ctx.endInit();
 		Solver solver2;
 		ctx.attach(solver2);
 		CPPUNIT_ASSERT(solver2.numConstraints() == 1);
