@@ -45,18 +45,24 @@ namespace ProgramOptions {
 std::size_t DefaultFormat::format(std::vector<char>& buf, const Option& o, std::size_t maxW) {
 	buf.clear();
 	size_t bufSize = std::max(maxW, o.maxColumn()) + 3;
+	const char* arg= o.argName();
+	const char* np = "";
+	const char* ap = "";
+	if (o.value()->isNegatable()) {
+		if (!*arg) { np = "[no-]"; }
+		else       { ap = "|no"; bufSize += strlen(ap); }
+	}
 	buf.resize(bufSize);
 	char*  buffer  = &buf[0];
-	size_t n       = sprintf(buffer, "  --%s", o.name().c_str());
-	const char* arg= o.argName();
+	size_t n       = sprintf(buffer, "  --%s%s", np, o.name().c_str());
 	if (o.value()->isImplicit() && *arg) {
-		n += sprintf(buffer+n, "[=%s]", arg);
+		n += sprintf(buffer+n, "[=%s%s]", arg, ap);
 	}
 	if (o.alias()) {
 		n += sprintf(buffer+n, ",-%c", o.alias());
 	}
 	if (!o.value()->isImplicit()) {
-		n += sprintf(buffer+n, "%c%s", (!o.alias()?'=':' '), arg);
+		n += sprintf(buffer+n, "%c%s%s", (!o.alias()?'=':' '), arg, ap);
 	}
 	if (n < maxW) n += sprintf(buffer+n, "%-*.*s", int(maxW-n), int(maxW-n), " ");
 	assert(n <= bufSize);
@@ -166,7 +172,7 @@ const char* Value::desc(DescType t) const {
 const char* Value::implicit() const {
 	if (!hasProperty(property_implicit)) return 0;
 	const char* x = desc(desc_implicit);
-	return x ? x : "";
+	return x ? x : "1";
 }
 
 bool Value::parse(const std::string& name, const std::string& value, State st) {
@@ -202,6 +208,12 @@ std::size_t Option::maxColumn() const {
 		if (value()->isImplicit()) {
 			col += 2; // []
 		}
+		if (value()->isNegatable()) {
+			col += 3; // |no
+		}
+	}
+	else if (value()->isNegatable()) {
+		col += 5; // [no-]
 	}
 	return col;
 }
@@ -215,7 +227,7 @@ bool Option::assignDefault() const {
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionGroup
 ///////////////////////////////////////////////////////////////////////////////
-OptionGroup::OptionGroup(const std::string& caption) : caption_(caption), hide_(false) {}
+OptionGroup::OptionGroup(const std::string& caption, DescriptionLevel hl) : caption_(caption), level_(hl) {}
 OptionGroup::~OptionGroup() { }
 
 OptionInitHelper OptionGroup::addOptions() {
@@ -227,17 +239,21 @@ void OptionGroup::addOption(std::auto_ptr<Option> option) {
 	options_.push_back(opt);
 }
 
-std::size_t OptionGroup::maxColumn() const {
+std::size_t OptionGroup::maxColumn(DescriptionLevel level) const {
 	std::size_t maxW = 0;
 	for (option_iterator it = options_.begin(), end = options_.end(); it != end; ++it) {
-		maxW = std::max(maxW, (*it)->maxColumn());
+		if ((*it)->descLevel() <= level) {
+			maxW = std::max(maxW, (*it)->maxColumn());
+		}
 	}
 	return maxW;
 }
 
-void OptionGroup::format(OptionOutput& out, size_t maxW) const {
+void OptionGroup::format(OptionOutput& out, size_t maxW, DescriptionLevel dl) const {
 	for (option_iterator it = options_.begin(), end = options_.end(); it != end; ++it) {
-		out.printOption(**it, maxW);
+		if ((*it)->descLevel() <= dl) {
+			out.printOption(**it, maxW);
+		}
 	}
 }
 ///////////////////////////////////////////////////////////////////////////////
@@ -248,7 +264,7 @@ OptionInitHelper::OptionInitHelper(OptionGroup& owner)
 
 OptionInitHelper& OptionInitHelper::operator()(const char* name, Value* val, const char* desc) {
 	std::auto_ptr<Value> value(val);
-	if (!name || !*name) {
+	if (!name || !*name || *name == ',' || *name == '!') {
 		throw Error("Invalid empty option name");
 	}
 	const char* n = strchr(name, ',');
@@ -258,12 +274,30 @@ OptionInitHelper& OptionInitHelper::operator()(const char* name, Value* val, con
 	}
 	else {
 		longName.assign(name, n);
-		++n;
-		if (!*n || *(n+1)) {
-			// empty or too long short name
-			throw Error(std::string("Invalid short name '").append(name).append("'"));
+		unsigned level = owner_->descLevel();
+		const char* x  = ++n;
+		if (*x && (!x[1] || x[1] == ',')) {
+			shortName = *x++;
+			x        += *x == ',';
 		}
-		shortName = *n;
+		if (*x == '@') {
+			++x; level = 0;
+			while (*x >= '0' && *x <= '9') {
+				level *= 10;
+				level += *x - '0';
+				++x;
+			}
+		}
+		if (!*n || *x || level > desc_level_hidden) {
+			throw Error(std::string("Invalid Key '").append(name).append("'"));
+		}
+		value->level(DescriptionLevel(level));
+	}
+	if (*(longName.end()-1) == '!') {
+		bool neg = *(longName.end()-2) != '\\';
+		longName.erase(longName.end()- (1+!neg), longName.end());
+		if (neg) value->negatable();
+		else     longName += '!';
 	}
 	owner_->addOption(auto_ptr<Option>(new Option(longName, shortName, desc, value.release())));
 	return *this;
@@ -271,13 +305,17 @@ OptionInitHelper& OptionInitHelper::operator()(const char* name, Value* val, con
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionContext
 ///////////////////////////////////////////////////////////////////////////////
-OptionContext::OptionContext(const std::string& cap)
-	: caption_(cap) {
+OptionContext::OptionContext(const std::string& cap, DescriptionLevel def)
+	: caption_(cap)
+	, descLevel_(def) {
 }
 OptionContext::~OptionContext()
 {}
 
-size_t OptionContext::findGroup(const std::string& name) const {
+void OptionContext::setActiveDescLevel(DescriptionLevel x) {
+	descLevel_ = std::min(x, desc_level_all);
+}
+size_t OptionContext::findGroupKey(const std::string& name) const {
 	for (size_t i = 0; i != groups_.size(); ++i) {
 		if (groups_[i].caption() == name) { return i; }
 	}
@@ -285,19 +323,29 @@ size_t OptionContext::findGroup(const std::string& name) const {
 }
 
 OptionContext& OptionContext::add(const OptionGroup& options) {
-	size_t k = findGroup(options.caption());
+	size_t k = findGroupKey(options.caption());
 	if (k >= groups_.size()) {
 		// add as new group
 		k = groups_.size();
-		groups_.push_back(OptionGroup(options.caption()));
+		groups_.push_back(OptionGroup(options.caption(), options.descLevel()));
 	}
 	for (option_iterator it = options.begin(), end = options.end(); it != end; ++it) {
 		insertOption(k, *it);
 	}
-	groups_[k].hidden( options.hidden() );
+	groups_[k].setDescriptionLevel(std::min(options.descLevel(), groups_[k].descLevel()));
 	return *this;
 }
 
+const OptionGroup& OptionContext::findGroup(const std::string& name) const {
+	std::size_t x = findGroupKey(name);
+	if (x < groups_.size()) { return groups_[x]; }
+	throw ContextError(caption(), ContextError::unknown_group, name);
+}
+const OptionGroup* OptionContext::tryFindGroup(const std::string& name) const {
+	std::size_t x = findGroupKey(name);
+	return x < groups_.size() ? &groups_[x] : 0;
+}
+
 OptionContext& OptionContext::add(const OptionContext& other) {
 	if (this == &other) return *this;
 	for (size_t g = 0; g != other.groups_.size(); ++g) {
@@ -306,7 +354,6 @@ OptionContext& OptionContext::add(const OptionContext& other) {
 	return *this;
 }
 
-
 void OptionContext::insertOption(size_t groupId, const SharedOptPtr& opt) {
 	const string& l = opt->name();
 	key_type k(options_.size());
@@ -372,49 +419,49 @@ OptionContext::PrefixRange OptionContext::findImpl(const char* key, FindType t)
 	return PrefixRange(index_.end(), index_.end()); 
 }
 
-OptionOutput& OptionContext::description(OptionOutput& out, bool addHidden) const {
-	size_t maxW = 23;
-	for (size_t i = 0; i != groups(); ++i) {
-		maxW = std::max(maxW, groups_[i].maxColumn());
-	}
-	if (out.printContext(*this)) {
-		// print all visible sub-groups
+OptionOutput& OptionContext::description(OptionOutput& out) const {
+	DescriptionLevel dl = descLevel_;
+	if (out.printContext(*this)) {		
+		size_t maxW = 23;
+		for (size_t i = 0; i != groups(); ++i) {
+			maxW = std::max(maxW, groups_[i].maxColumn(dl));
+		}
+		// print all visible groups
 		for (std::size_t i = 1; i < groups_.size(); ++i) {
-			if (!groups_[i].hidden() && out.printGroup(groups_[i])) {
-				groups_[i].format(out, maxW);
+			if (groups_[i].descLevel() <= dl && out.printGroup(groups_[i])) {
+				groups_[i].format(out, maxW, dl);
 			}
 		}
-		// print main group and all hidden sub-groups
-		for (std::size_t i = 0, end = addHidden ? groups_.size() : 0 + !groups_.empty(); i < end; ++i) {
-			if ((i == 0 || groups_[i].hidden()) && out.printGroup(groups_[i])) {
-				groups_[i].format(out, maxW);
-			}
+		if (!groups_.empty() && groups_[0].descLevel() <= dl && out.printGroup(groups_[0])) {
+			groups_[0].format(out, maxW, dl);
 		}
 	}
 	return out;
 }
 
 std::string OptionContext::defaults(std::size_t n) const {
-	std::size_t line = n, x = 0;
-	std::string defs; defs.reserve(options_.size());
+	DescriptionLevel dl = descLevel_;
+	std::size_t line    = n;
+	std::string defs; 
+	defs.reserve(options_.size());
+	std::string opt; opt.reserve(80);
 	for (int g = 0; g < 2; ++g) {
 		// print all sub-groups followed by main group
 		for (std::size_t i = (g == 0), end = (g == 0) ? groups_.size() : 1; i < end; ++i) {
-			if (!groups_[i].hidden()) {
+			if (groups_[i].descLevel() <= dl) {
 				for (option_iterator it = groups_[i].begin(), end = groups_[i].end(); it != end; ++it) {
 					const Option& o = **it;
-					if (o.value()->defaultsTo()) {
-						x = defs.size();
-						defs += "--";
-						defs += o.name();
-						defs += '=';
-						defs += o.value()->defaultsTo();
-						defs += ' ';
-						if ( (line += (defs.size() - x)) > 64) {
+					if (o.value()->defaultsTo() && o.descLevel() <= dl) {
+						((((opt += "--") += o.name()) += "=") += o.value()->defaultsTo());
+						if (line + opt.size() > 78) {
 							defs += '\n';
 							defs.append(n, ' ');
-							line = n;
+							line  = n;
 						}
+						defs += opt;
+						defs += ' ';
+						line += opt.size() + 1;
+						opt.clear();
 					}
 				}
 			}
@@ -424,7 +471,7 @@ std::string OptionContext::defaults(std::size_t n) const {
 }
 std::ostream& operator<<(std::ostream& os, const OptionContext& grp) {
 	StreamOut out(os);
-	grp.description(out, false);
+	grp.description(out);
 	return os;
 }
 
@@ -450,13 +497,18 @@ bool ParsedOptions::assign(const ParsedValues& p) {
 			begin = it = p.begin();
 			// assign parsed values
 			for (ParsedValues::iterator end = p.end(); it != end; ++it) {
-				const Option& o = *it->first;;
+				const Option& o = *it->first;
 				self->assign(o, it->second);
 			}
 		}
 		~Assign() { 
 			for (ParsedValues::iterator x = begin, end = this->it; x != end; ++x) {
-				self->parsed_.insert(x->first->name());
+				const Option& o = *x->first;
+				assert(o.value()->state() == Value::value_fixed || self->parsed_.count(o.name()) != 0);
+				if (o.value()->state() == Value::value_fixed) {
+					self->parsed_.insert(x->first->name());
+					o.value()->state(Value::value_unassigned);
+				}
 			}
 		}
 		ParsedOptions* self;
@@ -470,9 +522,9 @@ void ParsedOptions::assign(const Option& o, const std::string& value) {
 	unsigned badState = 0;
 	if (!o.value()->isComposing()) {
 		if (parsed_.count(o.name())) { return ; }
-		badState = (Value::value_parsed & o.value()->state());
+		badState = (Value::value_fixed & o.value()->state());
 	}
-	if (badState || !o.value()->parse(o.name(), value, Value::value_parsed)) {
+	if (badState || !o.value()->parse(o.name(), value, Value::value_fixed)) {
 		badState 
 			? throw ValueError("", ValueError::multiple_occurences, o.name(), value)
 			: throw ValueError("", ValueError::invalid_value, o.name(), value);
@@ -498,7 +550,7 @@ void ParsedValues::add(const std::string& name, const std::string& value) {
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionParser
 ///////////////////////////////////////////////////////////////////////////////
-OptionParser::OptionParser(OptionContext& o, bool allowUnreg)
+OptionParser::OptionParser(const OptionContext& o, bool allowUnreg)
 	: pv_(o)
 	, allowUnreg_(allowUnreg)
 {}
@@ -510,26 +562,12 @@ ParsedValues OptionParser::parse() {
 	doParse();
 	return pv_;
 }
-
-OptionParser::OptionType OptionParser::getOptionType(const char* o) const {
-	if (*o == '-' && *(o + 1) == '-') {
-		return  *(o + 2) != '\0' ? long_opt : end_opt;
-	}
-	return *o == '-' && *(o + 1) != '\0' ? short_opt : no_opt;
+SharedOptPtr OptionParser::tryGetOption(const char* name, FindType ft) const {
+	OptionContext::option_iterator it = pv_.ctx->tryFind(name, ft);
+	return it != pv_.ctx->end() ? *it : SharedOptPtr(0);
 }
-
-SharedOptPtr OptionParser::getOption(const char* name, OptionType t) const {
-	assert(t == short_opt || t == long_opt);
-	try {
-		OptionContext::FindType ft = (t == short_opt ? OptionContext::find_alias : OptionContext::find_name_or_prefix);
-		return *pv_.ctx->find(name, ft);
-	}
-	catch (const UnknownOption&) {
-		if (allowUnreg_) {
-			return SharedOptPtr(0);
-		}
-		throw;
-	}
+SharedOptPtr OptionParser::getOption(const char* name, FindType ft) const {
+	return !allowUnreg_ ? *pv_.ctx->find(name, ft) : tryGetOption(name, ft); 
 }
 namespace {   
 ///////////////////////////////////////////////////////////////////////////////
@@ -537,7 +575,8 @@ namespace {
 ///////////////////////////////////////////////////////////////////////////////    
 class CommandLineParser : public OptionParser {
 public:
-	CommandLineParser(OptionContext& o, bool allowUnreg, PosOption po)
+	enum OptionType {short_opt, neg_opt, long_opt, end_opt, no_opt};
+	CommandLineParser(const OptionContext& o, bool allowUnreg, PosOption po)
 		: OptionParser(o, allowUnreg)
 		, posOpt_(po)
 	{}
@@ -552,14 +591,15 @@ private:
 		while ( (curr=next()) && !breakEarly ) {
 			switch(getOptionType(curr)) {
 				case short_opt: if (handleShortOpt(curr + 1)) curr = 0; break;
+				case neg_opt :  if (handleNegOpt(curr+5)) { curr = 0; break; } // else: fall through
 				case long_opt:  if (handleLongOpt(curr + 2)) curr = 0;  break;
 				case end_opt:   curr = 0; breakEarly = true; break;
 				case no_opt:  
 				if (!posOpt_ || !posOpt_(curr, poName)) {
-					po = getOption("Positional Option", long_opt);
+					po = getOption("Positional Option", OptionContext::find_name_or_prefix);
 				}
 				else if (poName != poNameLast) {
-					po = getOption(poName.c_str(), long_opt);
+					po = getOption(poName.c_str(), OptionContext::find_name_or_prefix);
 					poNameLast = poName;
 				}
 				if (po.get())  {
@@ -579,6 +619,13 @@ private:
 			curr = next();
 		}
 	}
+	OptionType getOptionType(const char* o) const {
+		if (strncmp(o, "--", 2) == 0) {
+			if (strncmp(o+2, "no-", 3) == 0) return neg_opt;
+			else                             return o[2] ? long_opt : end_opt;
+		}
+		return *o == '-' && *(o + 1) != '\0' ? short_opt : no_opt;
+	}
 	bool handleShortOpt(const char* optName) {
 		// either -o value or -o[value|opts]
 		char optn[2];
@@ -587,7 +634,7 @@ private:
 		while (*optName) {
 			optn[0]         = *optName;
 			const char* val = optName + 1;
-			if ( (o = getOption(optn, short_opt)).get() ) {
+			if ( (o = getOption(optn, OptionContext::find_alias)).get() ) {
 				if (o->value()->isImplicit()) {
 					// -ovalue or -oopts
 					if (!o->value()->isFlag()) {
@@ -616,6 +663,14 @@ private:
 		}
 		return true;
 	}
+	bool handleNegOpt(const char* optName) {
+		SharedOptPtr o = tryGetOption(optName, OptionContext::find_name_or_prefix);
+		if (o.get() && o->value()->isNegatable()) {
+			addOptionValue(o, "no");
+			return true;
+		}
+		return false;
+	}
 	bool handleLongOpt(const char* optName) {
 		string name(optName);
 		string value;
@@ -625,7 +680,7 @@ private:
 			name.erase(p, string::npos);
 		}
 		SharedOptPtr o;
-		if ( (o = getOption(name.c_str(), long_opt)).get() ) {
+		if ( (o = getOption(name.c_str(), OptionContext::find_name_or_prefix)).get() ) {
 			if (!o->value()->isImplicit() && value.empty()) {
 				if (const char* v = next()) { value = v; }
 				else { throw SyntaxError(SyntaxError::missing_value, name); }
@@ -645,7 +700,7 @@ private:
 
 class ArgvParser : public CommandLineParser {
 public:
-	ArgvParser(OptionContext& o, bool allowUnreg, int startPos, char** argv, PosOption po)
+	ArgvParser(const OptionContext& o, bool allowUnreg, int startPos, char** argv, PosOption po)
 		: CommandLineParser(o, allowUnreg, po)
 		, currentArg_(0)
 		, argPos_(startPos)
@@ -664,7 +719,7 @@ private:
 
 class CommandStringParser : public CommandLineParser {
 public:
-	CommandStringParser(const std::string& cmd, OptionContext& o, bool allowUnreg, PosOption po)
+	CommandStringParser(const std::string& cmd, const OptionContext& o, bool allowUnreg, PosOption po)
 		: CommandLineParser(o, allowUnreg, po)
 		, cmd_(cmd)
 		, pos_(0)
@@ -694,7 +749,7 @@ private:
 class CfgFileParser : public OptionParser
 {
 public:
-	CfgFileParser(OptionContext& o, bool allowUnreg, std::istream& in)
+	CfgFileParser(const OptionContext& o, bool allowUnreg, std::istream& in)
 		: OptionParser(o, allowUnreg)
 		, in_(in)
 	{}
@@ -725,6 +780,7 @@ private:
 		std::string sectionName;      // current section name
 		std::string sectionValue;     // current section value
 		bool inSection = false;       // true if multi line section value
+		FindType ft    = OptionContext::find_name_or_prefix;
 		SharedOptPtr opt;
 		// reads the config file.
 		// A config file may only contain empty lines, single line comments or
@@ -739,7 +795,7 @@ private:
 			if (line.empty() || line.find("#") == 0) {
 				// An empty line or single line comment stops a multi line section value.
 				if (inSection) {
-					if ( (opt = getOption(sectionName.c_str(), long_opt)).get() )
+					if ( (opt = getOption(sectionName.c_str(), ft)).get() )
 						addOptionValue(opt, sectionValue);
 					inSection = false;
 				}
@@ -750,7 +806,7 @@ private:
 				// A new section terminates a multi line section value.
 				// First process the current section value...
 				if (inSection) {
-					if ( (opt = getOption(sectionName.c_str(), long_opt)).get() )
+					if ( (opt = getOption(sectionName.c_str(), ft)).get() )
 						addOptionValue(opt, sectionValue);
 					inSection = false;
 				}
@@ -769,7 +825,7 @@ private:
 			}
 		}
 		if (inSection) { // file does not end with an empty line
-			if ( (opt = getOption(sectionName.c_str(), long_opt)).get() )
+			if ( (opt = getOption(sectionName.c_str(), ft)).get() )
 				addOptionValue(opt, sectionValue);
 		}
 	}
@@ -778,7 +834,7 @@ private:
 
 } // end unnamed namespace
 
-ParsedValues parseCommandLine(int& argc, char** argv, OptionContext& o, bool allowUnreg, PosOption po) {
+ParsedValues parseCommandLine(int& argc, char** argv, const OptionContext& o, bool allowUnreg, PosOption po) {
 	ArgvParser pa(o, allowUnreg, 1, argv, po);
 	ParsedValues ret = pa.parse();
 	argc = 1 + (int)pa.remaining.size();
@@ -789,12 +845,12 @@ ParsedValues parseCommandLine(int& argc, char** argv, OptionContext& o, bool all
 	return ret;
 }
 
-ParsedValues parseCommandString(const std::string& cmd, OptionContext& o, bool allowUnreg, PosOption po) {
+ParsedValues parseCommandString(const std::string& cmd, const OptionContext& o, bool allowUnreg, PosOption po) {
 	CommandStringParser parser(cmd, o, allowUnreg, po);
 	return parser.parse();
 }
 
-ParsedValues parseCfgFile(std::istream& in, OptionContext& o, bool allowUnreg) {
+ParsedValues parseCfgFile(std::istream& in, const OptionContext& o, bool allowUnreg) {
 	CfgFileParser p(o, allowUnreg, in);
 	return p.parse();
 }
@@ -823,6 +879,7 @@ static std::string format(ContextError::Type t, const std::string& ctx, const st
 		case ContextError::duplicate_option: ret += "duplicate option: "; break;
 		case ContextError::unknown_option:   ret += "unknown option: ";   break;
 		case ContextError::ambiguous_option: ret += "ambiguous option: "; break;
+		case ContextError::unknown_group:    ret += "unknown group: ";    break;
 		default:                             ret += "unknown error in: ";
 	};
 	ret += quote(key);
